/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  Pseudo_QC_String.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_crypto.h"
#include "qore/intern/QoreLibIntern.h"

#include <ctype.h>
/* Qore class Qore::<string> */

qore_classid_t CID_PSEUDOSTRING;
QoreClass* QC_PSEUDOSTRING;

// int PseudoString::comparePartial(string ostr){}
static QoreValue PseudoString_comparePartial_Vs(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* ostr = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 811 "Pseudo_QC_String.qpp"
    // convert pattern encoding to string if necessary
   TempEncodingHelper temp(ostr, str->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();

   int rc = strncmp(str->getBuffer(), temp->getBuffer(), temp->size());
   return !rc ? 0 : (rc < 0 ? -1 : 1);
}

// bool PseudoString::empty(){}
static QoreValue PseudoString_empty(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 132 "Pseudo_QC_String.qpp"
    return str->empty();
}

// string PseudoString::encoding(){}
static QoreValue PseudoString_encoding(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 147 "Pseudo_QC_String.qpp"
    return new QoreStringNode(str->getEncoding()->getCode());
}

// bool PseudoString::equalPartial(string ostr){}
static QoreValue PseudoString_equalPartial_Vs(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* ostr = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 835 "Pseudo_QC_String.qpp"
    return str->equalPartialSoft(ostr, xsink);
}

// bool PseudoString::equalPartialPath(string ostr){}
static QoreValue PseudoString_equalPartialPath_Vs(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* ostr = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 851 "Pseudo_QC_String.qpp"
    return str->equalPartialPath(ostr, xsink);
}

// int PseudoString::find(softstring substr, softint pos = 0){}
static QoreValue PseudoString_find_vsvi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 0);
    int64 pos = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 235 "Pseudo_QC_String.qpp"
    return str->index(*substr, pos, xsink);
}

// string PseudoString::getDecoded(int code = CD_ALL){}
static QoreValue PseudoString_getDecoded_Vi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 1027 "Pseudo_QC_String.qpp"
    QoreStringNodeHolder rv(new QoreStringNode(str->getEncoding()));
   return rv->concatDecode(xsink, *str, code) ? 0 : rv.release();
}

// string PseudoString::getEncoded(int code = CE_XHTML){}
static QoreValue PseudoString_getEncoded_Vi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 code = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 1010 "Pseudo_QC_String.qpp"
    QoreStringNodeHolder rv(new QoreStringNode(str->getEncoding()));
   return rv->concatEncode(xsink, *str, code) ? 0 : rv.release();
}

// *string PseudoString::getLine(int offset = 0, *string eol, bool trim = True, *reference<int> size){}
static QoreValue PseudoString_getLine_ViNsVbC15_reference_int_(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 offset = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* eol = get_param_value(args, 1).get<const QoreStringNode>();
    bool trim = HARD_QORE_VALUE_BOOL(args, 2);
    const ReferenceNode* size = get_param_value(args, 3).get<const ReferenceNode>();
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 918 "Pseudo_QC_String.qpp"
    if (str->empty())
      return QoreValue();

   if (offset < 0) {
      offset = str->size() + offset;
      if (offset < 0)
         offset = 0;
   }
   else if (offset >= (int64)str->size())
      return QoreValue();

   TempEncodingHelper eolstr;
   if (eol) {
      eolstr.set(eol, str->getEncoding(), xsink);
      if (*xsink)
         return QoreValue();

      if (eolstr->size() > str->size())
         return QoreValue();
   }

   int64 strsize = 0;

   const char* start = str->getBuffer() + offset;
   SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (eolstr) {
      const char* p = strstr(start, eolstr->getBuffer());
      if (p) {
         strsize = p - start + eolstr->size();
         if (trim) {
            if (p != start)
               rv->concat(start, p - start);
         }
         else
            rv->concat(start, strsize);
      }
      else {
         strsize = rv->size();
         rv->concat(start);
         strsize = rv->size() - strsize;
      }
   }
   else {
      while (*start) {
         if (*start == '\r') {
            ++strsize;
            if (!trim)
               rv->concat(*start);
            if (*(start + 1) == '\n') {
               ++strsize;
               if (!trim)
                  rv->concat(*start);
            }
            break;
         }
         if (*start == '\n') {
            ++strsize;
            if (!trim)
               rv->concat(*start);
            break;
         }
         rv->concat(*start);
         ++strsize;
         ++start;
      }
   }

   if (size) {
      QoreTypeSafeReferenceHelper r(size, xsink);
      if (!r)
         return QoreValue();

      if (r.assign(strsize))
         return QoreValue();
   }

   return rv.release();
}

// int PseudoString::getUnicode(int offset = 0){}
static QoreValue PseudoString_getUnicode_Vi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 offset = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 888 "Pseudo_QC_String.qpp"
    return str->getUnicodePoint(offset, xsink);
}

// bool PseudoString::intp(){}
static QoreValue PseudoString_intp(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 291 "Pseudo_QC_String.qpp"
    if (str->empty())
      return false;
   char c = (*str)[0];
   if (c == '-')
      c = (*str)[1];
   return (isdigit(c)) ? true : false;
}

// bool PseudoString::isDataAscii(){}
static QoreValue PseudoString_isDataAscii(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 737 "Pseudo_QC_String.qpp"
    return str->isDataAscii();
}

// bool PseudoString::isDataPrintableAscii(){}
static QoreValue PseudoString_isDataPrintableAscii(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 751 "Pseudo_QC_String.qpp"
    return str->isDataPrintableAscii();
}

// int PseudoString::length(){}
static QoreValue PseudoString_length(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 118 "Pseudo_QC_String.qpp"
    return str->length();
}

// string PseudoString::lwr(){}
static QoreValue PseudoString_lwr(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 172 "Pseudo_QC_String.qpp"
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_tolower(*(*rv), *str, xsink))
      return QoreValue();

   return rv.release();
}

// bool PseudoString::regex(string regex, int options = 0){}
static QoreValue PseudoString_regex_VsVi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 0);
    int64 options = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 483 "Pseudo_QC_String.qpp"
    QoreRegex qr(*regex, options, xsink);
   if (*xsink)
      return QoreValue();

   return qr.exec(str, xsink);
}

// *list<string> PseudoString::regexExtract(string regex, int options = 0){}
static QoreValue PseudoString_regexExtract_VsVi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* regex = HARD_QORE_VALUE_STRING(args, 0);
    int64 options = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 517 "Pseudo_QC_String.qpp"
    QoreRegex qr(*regex, options, xsink);
   if (*xsink)
      return QoreValue();

   return qr.extractSubstrings(str, xsink);
}

// int PseudoString::rfind(softstring substr, softint pos = -1){}
static QoreValue PseudoString_rfind_vsvi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* substr = HARD_QORE_VALUE_STRING(args, 0);
    int64 pos = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 265 "Pseudo_QC_String.qpp"
    return str->rindex(*substr, (qore_offset_t)pos, xsink);
}

// int PseudoString::size(){}
static QoreValue PseudoString_size(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 97 "Pseudo_QC_String.qpp"
    return str->size();
}

// bool PseudoString::sizep(){}
static QoreValue PseudoString_sizep(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 899 "Pseudo_QC_String.qpp"
    return true;
}

// list<string> PseudoString::split(string sep, bool with_separator = False){}
static QoreValue PseudoString_split_VsVb(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sep = HARD_QORE_VALUE_STRING(args, 0);
    bool with_separator = HARD_QORE_VALUE_BOOL(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 422 "Pseudo_QC_String.qpp"
    // convert pattern encoding to string if necessary
   TempEncodingHelper temp(sep, str->getEncoding(), xsink);
   if (*xsink)
      return QoreValue();

   return split_intern(temp->getBuffer(), temp->strlen(), str->getBuffer(), str->strlen(), str->getEncoding(), with_separator);
}

// list<string> PseudoString::split(string sep, string quote, bool trim_unquoted = False){}
static QoreValue PseudoString_split_VsVsVb(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sep = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* quote = HARD_QORE_VALUE_STRING(args, 1);
    bool trim_unquoted = HARD_QORE_VALUE_BOOL(args, 2);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 456 "Pseudo_QC_String.qpp"
    return split_with_quote(sep, str, quote, trim_unquoted, xsink);
}

// int PseudoString::strlen(){}
static QoreValue PseudoString_strlen(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 76 "Pseudo_QC_String.qpp"
    return str->strlen();
}

// bool PseudoString::strp(){}
static QoreValue PseudoString_strp(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 348 "Pseudo_QC_String.qpp"
    return true;
}

// string PseudoString::substr(softint start){}
static QoreValue PseudoString_substr_vi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 369 "Pseudo_QC_String.qpp"
    QoreStringNode* rv = str->substr(start, xsink);
   if (!rv)
      rv = new QoreStringNode(str->getEncoding());
   return rv;
}

// string PseudoString::substr(softint start, softint len){}
static QoreValue PseudoString_substr_vivi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
    int64 len = HARD_QORE_VALUE_INT(args, 1);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 396 "Pseudo_QC_String.qpp"
    QoreStringNode* rv = str->substr(start, len, xsink);
   if (!rv)
      rv = new QoreStringNode(str->getEncoding());
   return rv;
}

// string PseudoString::toBase64(softint maxlinelen = -1){}
static QoreValue PseudoString_toBase64_vi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 maxlinelen = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 793 "Pseudo_QC_String.qpp"
    QoreStringNode *rv = new QoreStringNode;
   rv->concatBase64(str, maxlinelen);
   return rv;
}

// string PseudoString::toHex(){}
static QoreValue PseudoString_toHex(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 769 "Pseudo_QC_String.qpp"
    QoreStringNode *rv = new QoreStringNode;
   rv->concatHex(str);
   return rv;
}

// int PseudoString::toInt(int base = 10){}
static QoreValue PseudoString_toInt_Vi(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 base = HARD_QORE_VALUE_INT(args, 0);
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 330 "Pseudo_QC_String.qpp"
    if (base < 0 || base == 1 || base > 36)
        return xsink -> raiseException("INVALID-BASE", "base " QLLD " is invalid; base must be 0 or 2 - 36 inclusive", base);
    if (!str->getEncoding()->isAsciiCompat())
        return xsink -> raiseException("UNSUPPORTED-ENCODING", "cannot convert string in non-ASCII-compatible encoding \"%s\" to an integer", str->getEncoding()->getCode());
    return strtoll(str -> getBuffer(), 0, (int)base);
}

// string PseudoString::toMD5(){}
static QoreValue PseudoString_toMD5(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 547 "Pseudo_QC_String.qpp"
    DigestHelper dh(*str);
   if (dh.doDigest(MD5_ERR, EVP_md5(), xsink))
      return QoreValue();

   return dh.getString();
}

// string PseudoString::toSHA1(){}
static QoreValue PseudoString_toSHA1(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 575 "Pseudo_QC_String.qpp"
    DigestHelper dh(*str);
   if (dh.doDigest(SHA1_ERR, EVP_sha1(), xsink))
      return QoreValue();

   return dh.getString();
}

// string PseudoString::toSHA224(){}
static QoreValue PseudoString_toSHA224(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 606 "Pseudo_QC_String.qpp"
    #if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*str);
   if (dh.doDigest(SHA224_ERR, EVP_sha224(), xsink))
      return QoreValue();

   return dh.getString();
#else
   missing_openssl_feature("SHA224", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

// string PseudoString::toSHA256(){}
static QoreValue PseudoString_toSHA256(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 642 "Pseudo_QC_String.qpp"
    #if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*str);
   if (dh.doDigest(SHA256_ERR, EVP_sha256(), xsink))
      return QoreValue();

   return dh.getString();
#else
   missing_openssl_feature("SHA256", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

// string PseudoString::toSHA384(){}
static QoreValue PseudoString_toSHA384(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 678 "Pseudo_QC_String.qpp"
    #if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*str);
   if (dh.doDigest(SHA384_ERR, EVP_sha384(), xsink))
      return QoreValue();

   return dh.getString();
#else
   missing_openssl_feature("SHA384", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

// string PseudoString::toSHA512(){}
static QoreValue PseudoString_toSHA512(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 714 "Pseudo_QC_String.qpp"
    #if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
   DigestHelper dh(*str);
   if (dh.doDigest(SHA512_ERR, EVP_sha512(), xsink))
      return QoreValue();

   return dh.getString();
#else
   missing_openssl_feature("SHA512", xsink);
   return QoreValue();
#endif
    return QoreValue();
}

// int PseudoString::typeCode(){}
static QoreValue PseudoString_typeCode(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 57 "Pseudo_QC_String.qpp"
    return NT_STRING;
}

// string PseudoString::unaccent(){}
static QoreValue PseudoString_unaccent(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 868 "Pseudo_QC_String.qpp"
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_unaccent(*(*rv), *str, xsink))
      return QoreValue();

   return rv.release();
}

// string PseudoString::upr(){}
static QoreValue PseudoString_upr(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 201 "Pseudo_QC_String.qpp"
    SimpleRefHolder<QoreStringNode> rv(new QoreStringNode(str->getEncoding()));
   if (do_toupper(*(*rv), *str, xsink))
      return QoreValue();

   return rv.release();
}

// bool PseudoString::val(){}
static QoreValue PseudoString_val(QoreObject* self, const QoreValue& v, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* str = v.get<const QoreStringNode>();
# 277 "Pseudo_QC_String.qpp"
    return !str->empty();
}

DLLLOCAL void preinitPseudoStringClass() {
    QC_PSEUDOSTRING = new QoreClass("<string>", QDOM_DEFAULT);
    CID_PSEUDOSTRING = QC_PSEUDOSTRING->getID();
    QC_PSEUDOSTRING->setSystem();
}

DLLLOCAL QoreClass* initPseudoStringClass() {
    if (!QC_PSEUDOSTRING)
        preinitPseudoStringClass();

    // set parent class
    assert(QC_PSEUDOVALUE);
    QC_PSEUDOSTRING->addBuiltinVirtualBaseClass(QC_PSEUDOVALUE);

    // int PseudoString::comparePartial(string ostr){}
    QC_PSEUDOSTRING->addMethod("comparePartial", (q_method_n_t)PseudoString_comparePartial_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 1, stringTypeInfo, NULL, "ostr");

    // bool PseudoString::empty(){}
    QC_PSEUDOSTRING->addMethod("empty", (q_method_n_t)PseudoString_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string PseudoString::encoding(){}
    QC_PSEUDOSTRING->addMethod("encoding", (q_method_n_t)PseudoString_encoding, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool PseudoString::equalPartial(string ostr){}
    QC_PSEUDOSTRING->addMethod("equalPartial", (q_method_n_t)PseudoString_equalPartial_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "ostr");

    // bool PseudoString::equalPartialPath(string ostr){}
    QC_PSEUDOSTRING->addMethod("equalPartialPath", (q_method_n_t)PseudoString_equalPartialPath_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "ostr");

    // int PseudoString::find(softstring substr, softint pos = 0){}
    QC_PSEUDOSTRING->addMethod("find", (q_method_n_t)PseudoString_find_vsvi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, softStringTypeInfo, NULL, "substr", softBigIntTypeInfo, zero(), "pos");

    // string PseudoString::getDecoded(int code = CD_ALL){}
    QC_PSEUDOSTRING->addMethod("getDecoded", (q_method_n_t)PseudoString_getDecoded_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, bigIntTypeInfo, new QoreBigIntNode(CD_ALL), "code");

    // string PseudoString::getEncoded(int code = CE_XHTML){}
    QC_PSEUDOSTRING->addMethod("getEncoded", (q_method_n_t)PseudoString_getEncoded_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, bigIntTypeInfo, new QoreBigIntNode(CE_XHTML), "code");

    // *string PseudoString::getLine(int offset = 0, *string eol, bool trim = True, *reference<int> size){}
    QC_PSEUDOSTRING->addMethod("getLine", (q_method_n_t)PseudoString_getLine_ViNsVbC15_reference_int_, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo, 4, bigIntTypeInfo, zero(), "offset", stringOrNothingTypeInfo, NULL, "eol", boolTypeInfo, &True, "trim", qore_get_complex_reference_or_nothing_type(bigIntTypeInfo), NULL, "size");

    // int PseudoString::getUnicode(int offset = 0){}
    QC_PSEUDOSTRING->addMethod("getUnicode", (q_method_n_t)PseudoString_getUnicode_Vi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, zero(), "offset");

    // bool PseudoString::intp(){}
    QC_PSEUDOSTRING->addMethod("intp", (q_method_n_t)PseudoString_intp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoString::isDataAscii(){}
    QC_PSEUDOSTRING->addMethod("isDataAscii", (q_method_n_t)PseudoString_isDataAscii, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool PseudoString::isDataPrintableAscii(){}
    QC_PSEUDOSTRING->addMethod("isDataPrintableAscii", (q_method_n_t)PseudoString_isDataPrintableAscii, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int PseudoString::length(){}
    QC_PSEUDOSTRING->addMethod("length", (q_method_n_t)PseudoString_length, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // string PseudoString::lwr(){}
    QC_PSEUDOSTRING->addMethod("lwr", (q_method_n_t)PseudoString_lwr, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool PseudoString::regex(string regex, int options = 0){}
    QC_PSEUDOSTRING->addMethod("regex", (q_method_n_t)PseudoString_regex_VsVi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 2, stringTypeInfo, NULL, "regex", bigIntTypeInfo, zero(), "options");

    // *list<string> PseudoString::regexExtract(string regex, int options = 0){}
    QC_PSEUDOSTRING->addMethod("regexExtract", (q_method_n_t)PseudoString_regexExtract_VsVi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_or_nothing_type(stringTypeInfo), 2, stringTypeInfo, NULL, "regex", bigIntTypeInfo, zero(), "options");

    // int PseudoString::rfind(softstring substr, softint pos = -1){}
    QC_PSEUDOSTRING->addMethod("rfind", (q_method_n_t)PseudoString_rfind_vsvi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 2, softStringTypeInfo, NULL, "substr", softBigIntTypeInfo, new QoreBigIntNode(-1), "pos");

    // int PseudoString::size(){}
    QC_PSEUDOSTRING->addMethod("size", (q_method_n_t)PseudoString_size, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoString::sizep(){}
    QC_PSEUDOSTRING->addMethod("sizep", (q_method_n_t)PseudoString_sizep, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // list<string> PseudoString::split(string sep, bool with_separator = False){}
    QC_PSEUDOSTRING->addMethod("split", (q_method_n_t)PseudoString_split_VsVb, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 2, stringTypeInfo, NULL, "sep", boolTypeInfo, &False, "with_separator");

    // list<string> PseudoString::split(string sep, string quote, bool trim_unquoted = False){}
    QC_PSEUDOSTRING->addMethod("split", (q_method_n_t)PseudoString_split_VsVsVb, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo), 3, stringTypeInfo, NULL, "sep", stringTypeInfo, NULL, "quote", boolTypeInfo, &False, "trim_unquoted");

    // int PseudoString::strlen(){}
    QC_PSEUDOSTRING->addMethod("strlen", (q_method_n_t)PseudoString_strlen, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool PseudoString::strp(){}
    QC_PSEUDOSTRING->addMethod("strp", (q_method_n_t)PseudoString_strp, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // string PseudoString::substr(softint start){}
    QC_PSEUDOSTRING->addMethod("substr", (q_method_n_t)PseudoString_substr_vi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, NULL, "start");

    // string PseudoString::substr(softint start, softint len){}
    QC_PSEUDOSTRING->addMethod("substr", (q_method_n_t)PseudoString_substr_vivi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 2, softBigIntTypeInfo, NULL, "start", softBigIntTypeInfo, NULL, "len");

    // string PseudoString::toBase64(softint maxlinelen = -1){}
    QC_PSEUDOSTRING->addMethod("toBase64", (q_method_n_t)PseudoString_toBase64_vi, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo, 1, softBigIntTypeInfo, new QoreBigIntNode(-1), "maxlinelen");

    // string PseudoString::toHex(){}
    QC_PSEUDOSTRING->addMethod("toHex", (q_method_n_t)PseudoString_toHex, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int PseudoString::toInt(int base = 10){}
    QC_PSEUDOSTRING->addMethod("toInt", (q_method_n_t)PseudoString_toInt_Vi, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo, 1, bigIntTypeInfo, new QoreBigIntNode(10), "base");

    // string PseudoString::toMD5(){}
    QC_PSEUDOSTRING->addMethod("toMD5", (q_method_n_t)PseudoString_toMD5, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::toSHA1(){}
    QC_PSEUDOSTRING->addMethod("toSHA1", (q_method_n_t)PseudoString_toSHA1, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::toSHA224(){}
    QC_PSEUDOSTRING->addMethod("toSHA224", (q_method_n_t)PseudoString_toSHA224, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::toSHA256(){}
    QC_PSEUDOSTRING->addMethod("toSHA256", (q_method_n_t)PseudoString_toSHA256, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::toSHA384(){}
    QC_PSEUDOSTRING->addMethod("toSHA384", (q_method_n_t)PseudoString_toSHA384, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::toSHA512(){}
    QC_PSEUDOSTRING->addMethod("toSHA512", (q_method_n_t)PseudoString_toSHA512, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // int PseudoString::typeCode(){}
    QC_PSEUDOSTRING->addMethod("typeCode", (q_method_n_t)PseudoString_typeCode, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // string PseudoString::unaccent(){}
    QC_PSEUDOSTRING->addMethod("unaccent", (q_method_n_t)PseudoString_unaccent, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // string PseudoString::upr(){}
    QC_PSEUDOSTRING->addMethod("upr", (q_method_n_t)PseudoString_upr, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // bool PseudoString::val(){}
    QC_PSEUDOSTRING->addMethod("val", (q_method_n_t)PseudoString_val, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_PSEUDOSTRING;
}
