/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractDatasource.qpp AbstractDatasource class (interface) definition */
/*
  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>

class AbstractDatasource {};
/* Qore class Qore::SQL::AbstractDatasource */

qore_classid_t CID_ABSTRACTDATASOURCE;
QoreClass* QC_ABSTRACTDATASOURCE;

// bool AbstractDatasource::currentThreadInTransaction(){}
static QoreValue AbstractDatasource_currentThreadInTransaction(QoreObject* self, AbstractDatasource* ds, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 413 "QC_AbstractDatasource.qpp"
    xsink->raiseException("ABSTRACT-DATASOURCE-ERROR", "This method must be re-implemented in subclasses to provide the desired functionality; it is implemented in Datasource and DatasourcePool for example");
   return false;
}

DLLLOCAL void preinitAbstractDatasourceClass() {
    QC_ABSTRACTDATASOURCE = new QoreClass("AbstractDatasource", QDOM_DATABASE);
    CID_ABSTRACTDATASOURCE = QC_ABSTRACTDATASOURCE->getID();
    QC_ABSTRACTDATASOURCE->setSystem();
}

DLLLOCAL QoreClass* initAbstractDatasourceClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTDATASOURCE)
        preinitAbstractDatasourceClass();

    // nothing AbstractDatasource::beginTransaction();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("beginTransaction", false, QC_NO_FLAGS, nothingTypeInfo);

    // nothing AbstractDatasource::commit();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("commit", false, QC_NO_FLAGS, nothingTypeInfo);

    // bool AbstractDatasource::currentThreadInTransaction(){}
    QC_ABSTRACTDATASOURCE->addMethod("currentThreadInTransaction", (q_method_n_t)AbstractDatasource_currentThreadInTransaction, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // auto AbstractDatasource::exec(string sql, ...);
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("exec", false, QC_USES_EXTRA_ARGS, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // auto AbstractDatasource::execRaw(string sql);
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("execRaw", false, QC_NO_FLAGS, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // auto AbstractDatasource::getClientVersion();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getClientVersion", false, QC_NO_FLAGS, autoTypeInfo);

    // hash AbstractDatasource::getConfigHash();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getConfigHash", false, QC_NO_FLAGS, hashTypeInfo);

    // string AbstractDatasource::getConfigString();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getConfigString", false, QC_NO_FLAGS, stringTypeInfo);

    // string AbstractDatasource::getDBEncoding();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getDBEncoding", false, QC_NO_FLAGS, stringTypeInfo);

    // *string AbstractDatasource::getDBName();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getDBName", false, QC_NO_FLAGS, stringOrNothingTypeInfo);

    // string AbstractDatasource::getDriverName();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getDriverName", false, QC_NO_FLAGS, stringTypeInfo);

    // *string AbstractDatasource::getHostName();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getHostName", false, QC_NO_FLAGS, stringOrNothingTypeInfo);

    // *string AbstractDatasource::getOSEncoding();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getOSEncoding", false, QC_NO_FLAGS, stringOrNothingTypeInfo);

    // *string AbstractDatasource::getPassword();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getPassword", false, QC_NO_FLAGS, stringOrNothingTypeInfo);

    // *int AbstractDatasource::getPort();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getPort", false, QC_NO_FLAGS, bigIntOrNothingTypeInfo);

    // auto AbstractDatasource::getServerVersion();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getServerVersion", false, QC_NO_FLAGS, autoTypeInfo);

    // *string AbstractDatasource::getUserName();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("getUserName", false, QC_NO_FLAGS, stringOrNothingTypeInfo);

    // bool AbstractDatasource::inTransaction();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("inTransaction", false, QC_NO_FLAGS, boolTypeInfo);

    // nothing AbstractDatasource::rollback();
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("rollback", false, QC_NO_FLAGS, nothingTypeInfo);

    // auto AbstractDatasource::select(string sql, ...);
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("select", false, QC_USES_EXTRA_ARGS, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // auto AbstractDatasource::selectRow(string sql, ...);
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("selectRow", false, QC_USES_EXTRA_ARGS, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // auto AbstractDatasource::selectRows(string sql, ...);
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("selectRows", false, QC_USES_EXTRA_ARGS, autoTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // auto AbstractDatasource::vexec(string sql, *softlist vargs);
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("vexec", false, QC_NO_FLAGS, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    // auto AbstractDatasource::vselect(string sql, *softlist vargs);
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("vselect", false, QC_NO_FLAGS, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    // auto AbstractDatasource::vselectRow(string sql, *softlist vargs);
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("vselectRow", false, QC_NO_FLAGS, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    // auto AbstractDatasource::vselectRows(string sql, *softlist vargs);
    QC_ABSTRACTDATASOURCE->addAbstractMethodVariantExtended3("vselectRows", false, QC_NO_FLAGS, autoTypeInfo, 2, stringTypeInfo, NULL, "sql", softListOrNothingTypeInfo, NULL, "vargs");

    return QC_ABSTRACTDATASOURCE;
}
