/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_FileInputStream.qpp FileInputStream class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/FileInputStream.h"

extern qore_classid_t CID_FILEINPUTSTREAM;

static void FILEINPUTSTREAM_system_constructor(QoreObject* self, int fd, va_list args) {
   self->setPrivate(CID_FILEINPUTSTREAM, new FileInputStream(fd));
}
/* Qore class Qore::FileInputStream */

qore_classid_t CID_FILEINPUTSTREAM;
QoreClass* QC_FILEINPUTSTREAM;

// FileInputStream::constructor(string fileName, timeout timeout_ms = -1) {}
static void FileInputStream_constructor_VsVt(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* fileName = HARD_QORE_VALUE_STRING(args, 0);
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 1);
# 66 "QC_FileInputStream.qpp"
SimpleRefHolder<FileInputStream> fis(new FileInputStream(fileName, timeout_ms, xsink));
   self->setPrivate(CID_FILEINPUTSTREAM, fis.release());
}

// int FileInputStream::peek(){}
static QoreValue FileInputStream_peek(QoreObject* self, FileInputStream* is, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 113 "QC_FileInputStream.qpp"
    return is->peekHelper(xsink);
}

// *binary FileInputStream::read(int limit){}
static QoreValue FileInputStream_read_Vi(QoreObject* self, FileInputStream* is, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 limit = HARD_QORE_VALUE_INT(args, 0);
# 94 "QC_FileInputStream.qpp"
    return is->readHelper(limit, xsink);
}

DLLLOCAL void preinitFileInputStreamClass() {
    QC_FILEINPUTSTREAM = new QoreClass("FileInputStream", QDOM_FILESYSTEM);
    CID_FILEINPUTSTREAM = QC_FILEINPUTSTREAM->getID();
    QC_FILEINPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initFileInputStreamClass(QoreNamespace& ns) {
    if (!QC_FILEINPUTSTREAM)
        preinitFileInputStreamClass();

    // set parent class
    assert(QC_INPUTSTREAM);
    QC_FILEINPUTSTREAM->addBuiltinVirtualBaseClass(QC_INPUTSTREAM);

    // set system constructor
    QC_FILEINPUTSTREAM->setSystemConstructor(FILEINPUTSTREAM_system_constructor);

    QC_FILEINPUTSTREAM->setFinal();

    // FileInputStream::constructor(string fileName, timeout timeout_ms = -1) {}
    QC_FILEINPUTSTREAM->addConstructor(FileInputStream_constructor_VsVt, false, QC_NO_FLAGS, QDOM_DEFAULT, 2, stringTypeInfo, NULL, "fileName", timeoutTypeInfo, new QoreBigIntNode(-1), "timeout_ms");

    // int FileInputStream::peek(){}
    QC_FILEINPUTSTREAM->addMethod("peek", (q_method_n_t)FileInputStream_peek, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // *binary FileInputStream::read(int limit){}
    QC_FILEINPUTSTREAM->addMethod("read", (q_method_n_t)FileInputStream_read_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, binaryOrNothingTypeInfo, 1, bigIntTypeInfo, NULL, "limit");

    return QC_FILEINPUTSTREAM;
}
