/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_FileOutputStream.qpp FileOutputStream class definition */
/*
  Qore Programming Language

  Copyright (C) 2016 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/FileOutputStream.h"

extern qore_classid_t CID_FILEOUTPUTSTREAM;

static void FILEOUTPUTSTREAM_system_constructor(QoreObject* self, int fd, va_list args) {
   self->setPrivate(CID_FILEOUTPUTSTREAM, new FileOutputStream(fd));
}
/* Qore class Qore::FileOutputStream */

qore_classid_t CID_FILEOUTPUTSTREAM;
QoreClass* QC_FILEOUTPUTSTREAM;

// nothing FileOutputStream::close(){}
static QoreValue FileOutputStream_close(QoreObject* self, FileOutputStream* os, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 80 "QC_FileOutputStream.qpp"
    os->closeHelper(xsink);
    return QoreValue();
}

// FileOutputStream::constructor(string fileName, bool append = False, softint mode = 0644, *string encoding) {}
static void FileOutputStream_constructor_VsVbviNs(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* fileName = HARD_QORE_VALUE_STRING(args, 0);
    bool append = HARD_QORE_VALUE_BOOL(args, 1);
    int64 mode = HARD_QORE_VALUE_INT(args, 2);
    const QoreStringNode* encoding = get_param_value(args, 3).get<const QoreStringNode>();
# 68 "QC_FileOutputStream.qpp"
SimpleRefHolder<FileOutputStream> fos(new FileOutputStream(fileName, append, mode, encoding ? QEM.findCreate(encoding) : QCS_DEFAULT, xsink));
   self->setPrivate(CID_FILEOUTPUTSTREAM, fos.release());
}

// string FileOutputStream::getEncoding(){}
static QoreValue FileOutputStream_getEncoding(QoreObject* self, FileOutputStream* os, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 118 "QC_FileOutputStream.qpp"
    return new QoreStringNode(os->getEncoding());
}

// nothing FileOutputStream::write(binary data){}
static QoreValue FileOutputStream_write_Vy(QoreObject* self, FileOutputStream* os, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 0);
# 103 "QC_FileOutputStream.qpp"
    os->writeHelper(data, xsink);
    return QoreValue();
}

DLLLOCAL void preinitFileOutputStreamClass() {
    QC_FILEOUTPUTSTREAM = new QoreClass("FileOutputStream", QDOM_FILESYSTEM);
    CID_FILEOUTPUTSTREAM = QC_FILEOUTPUTSTREAM->getID();
    QC_FILEOUTPUTSTREAM->setSystem();
}

DLLLOCAL QoreClass* initFileOutputStreamClass(QoreNamespace& ns) {
    if (!QC_FILEOUTPUTSTREAM)
        preinitFileOutputStreamClass();

    // set parent class
    assert(QC_OUTPUTSTREAM);
    QC_FILEOUTPUTSTREAM->addBuiltinVirtualBaseClass(QC_OUTPUTSTREAM);

    // set system constructor
    QC_FILEOUTPUTSTREAM->setSystemConstructor(FILEOUTPUTSTREAM_system_constructor);

    QC_FILEOUTPUTSTREAM->setFinal();

    // nothing FileOutputStream::close(){}
    QC_FILEOUTPUTSTREAM->addMethod("close", (q_method_n_t)FileOutputStream_close, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // FileOutputStream::constructor(string fileName, bool append = False, softint mode = 0644, *string encoding) {}
    QC_FILEOUTPUTSTREAM->addConstructor(FileOutputStream_constructor_VsVbviNs, false, QC_NO_FLAGS, QDOM_DEFAULT, 4, stringTypeInfo, NULL, "fileName", boolTypeInfo, &False, "append", softBigIntTypeInfo, new QoreBigIntNode(0644), "mode", stringOrNothingTypeInfo, NULL, "encoding");

    // string FileOutputStream::getEncoding(){}
    QC_FILEOUTPUTSTREAM->addMethod("getEncoding", (q_method_n_t)FileOutputStream_getEncoding, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // nothing FileOutputStream::write(binary data){}
    QC_FILEOUTPUTSTREAM->addMethod("write", (q_method_n_t)FileOutputStream_write_Vy, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, binaryTypeInfo, NULL, "data");

    return QC_FILEOUTPUTSTREAM;
}
