/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_FtpClient.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2016 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include "qore/Qore.h"
#include "qore/intern/ssl_constants.h"
#include "qore/intern/QC_FtpClient.h"
#include "qore/intern/QC_Queue.h"
/* Qore class Qore::FtpClient */

qore_classid_t CID_FTPCLIENT;
QoreClass* QC_FTPCLIENT;

// nothing FtpClient::clearStats(){}
static QoreValue FtpClient_clearStats(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1071 "QC_FtpClient.qpp"
    f->clearStats();
    return QoreValue();
}

// nothing FtpClient::clearWarningQueue(){}
static QoreValue FtpClient_clearWarningQueue(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 993 "QC_FtpClient.qpp"
    f->clearWarningQueue(xsink);
    return QoreValue();
}

// nothing FtpClient::connect(){}
static QoreValue FtpClient_connect(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 159 "QC_FtpClient.qpp"
    f->connect(xsink);
    return QoreValue();
}

// FtpClient::constructor() {}
static void FtpClient_constructor(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 92 "QC_FtpClient.qpp"
self->setPrivate(CID_FTPCLIENT, new QoreFtpClientClass);
}

// FtpClient::constructor(string url) {}
static void FtpClient_constructor_Vs(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 109 "QC_FtpClient.qpp"
ReferenceHolder<QoreFtpClientClass> f(new QoreFtpClientClass(url, xsink), xsink);
   if (*xsink)
      return;

   self->setPrivate(CID_FTPCLIENT, f.release());
}

// FtpClient::copy() {}
static void FtpClient_copy(QoreObject* self, QoreObject* old, QoreFtpClientClass* f, ExceptionSink* xsink) {
# 121 "QC_FtpClient.qpp"
xsink->raiseException("FTPCLIENT-COPY-ERROR", "FtpClient objects cannot be copied");
}

// nothing FtpClient::cwd(string path){}
static QoreValue FtpClient_cwd_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 303 "QC_FtpClient.qpp"
    f->cwd(path->getBuffer(), xsink);
    return QoreValue();
}

// nothing FtpClient::del(string remote_path){}
static QoreValue FtpClient_del_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
# 539 "QC_FtpClient.qpp"
    f->del(remote_path->getBuffer(), xsink);
    return QoreValue();
}

// FtpClient::destructor() {}
static void FtpClient_destructor(QoreObject* self, QoreFtpClientClass* f, ExceptionSink* xsink) {
# 135 "QC_FtpClient.qpp"
// have to clear callbacks before destroying
   f->cleanup(xsink);
   f->deref(xsink);
}

// nothing FtpClient::disconnect(){}
static QoreValue FtpClient_disconnect(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 172 "QC_FtpClient.qpp"
    f->disconnect();
    return QoreValue();
}

// nothing FtpClient::get(string remote_path, *string local_path){}
static QoreValue FtpClient_get_VsNs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* local_path = get_param_value(args, 1).get<const QoreStringNode>();
# 401 "QC_FtpClient.qpp"
    f->get(remote_path->getBuffer(), local_path ? local_path->getBuffer() : 0, xsink);
    return QoreValue();
}

// nothing FtpClient::get(string remote_path, Qore::OutputStream os){}
static QoreValue FtpClient_get_VsC12OutputStream(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
    HARD_QORE_VALUE_OBJ_DATA(os, OutputStream, args, 1, CID_OUTPUTSTREAM, "FtpClient::get()", "Qore::OutputStream", xsink);
    if (*xsink)
        return 0;
# 426 "QC_FtpClient.qpp"
    SimpleRefHolder<OutputStream> osHolder(os);
   f->get(remote_path->getBuffer(), os, xsink);
    return QoreValue();
}

// binary FtpClient::getAsBinary(string remote_path){}
static QoreValue FtpClient_getAsBinary_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
# 519 "QC_FtpClient.qpp"
    return f->getAsBinary(remote_path->getBuffer(), xsink);
}

// string FtpClient::getAsString(string remote_path){}
static QoreValue FtpClient_getAsString_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
# 494 "QC_FtpClient.qpp"
    return f->getAsString(remote_path->getBuffer(), xsink);
}

// *string FtpClient::getHostName(){}
static QoreValue FtpClient_getHostName(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 691 "QC_FtpClient.qpp"
    const char *h = f->getHostName();
   return h ? new QoreStringNode(h) : 0;
}

// string FtpClient::getMode(){}
static QoreValue FtpClient_getMode(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 886 "QC_FtpClient.qpp"
    return new QoreStringNode(f->getMode());
}

// *string FtpClient::getPassword(){}
static QoreValue FtpClient_getPassword(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 677 "QC_FtpClient.qpp"
    const char *p = f->getPassword();
   return p ? new QoreStringNode(p) : 0;
}

// int FtpClient::getPort(){}
static QoreValue FtpClient_getPort(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 705 "QC_FtpClient.qpp"
    return f->getPort();
}

// *string FtpClient::getSSLCipherName(){}
static QoreValue FtpClient_getSSLCipherName(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 801 "QC_FtpClient.qpp"
    const char *str = f->getSSLCipherName();
   return str ? new QoreStringNode(str) : 0;
}

// *string FtpClient::getSSLCipherVersion(){}
static QoreValue FtpClient_getSSLCipherVersion(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 815 "QC_FtpClient.qpp"
    const char *str = f->getSSLCipherVersion();
   return str ? new QoreStringNode(str) : 0;
}

// int FtpClient::getTimeout(){}
static QoreValue FtpClient_getTimeout(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1080 "QC_FtpClient.qpp"
    return f->getTimeout();
}

// string FtpClient::getURL(){}
static QoreValue FtpClient_getURL(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 718 "QC_FtpClient.qpp"
    return f->getURL();
}

// hash FtpClient::getUsageInfo(){}
static QoreValue FtpClient_getUsageInfo(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1057 "QC_FtpClient.qpp"
    return f->getUsageInfo();
}

// *string FtpClient::getUserName(){}
static QoreValue FtpClient_getUserName(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 663 "QC_FtpClient.qpp"
    const char *u = f->getUserName();
   return u ? new QoreStringNode(u) : 0;
}

// bool FtpClient::isDataSecure(){}
static QoreValue FtpClient_isDataSecure(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 788 "QC_FtpClient.qpp"
    return f->isDataSecure();
}

// bool FtpClient::isSecure(){}
static QoreValue FtpClient_isSecure(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 774 "QC_FtpClient.qpp"
    return get_bool_node(f->isSecure());
}

// *string FtpClient::list(){}
static QoreValue FtpClient_list(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 194 "QC_FtpClient.qpp"
    return f->list(0, true, xsink);
}

// *string FtpClient::list(string path){}
static QoreValue FtpClient_list_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 219 "QC_FtpClient.qpp"
    return f->list(path->getBuffer(), true, xsink);
}

// nothing FtpClient::mkdir(string remote_path){}
static QoreValue FtpClient_mkdir_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
# 559 "QC_FtpClient.qpp"
    f->mkdir(remote_path->getBuffer(), xsink);
    return QoreValue();
}

// *string FtpClient::nlst(){}
static QoreValue FtpClient_nlst(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 241 "QC_FtpClient.qpp"
    return f->list(0, false, xsink);
}

// *string FtpClient::nlst(string path){}
static QoreValue FtpClient_nlst_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* path = HARD_QORE_VALUE_STRING(args, 0);
# 265 "QC_FtpClient.qpp"
    return f->list(path->getBuffer(), false, xsink);
}

// nothing FtpClient::put(string local_path, *string remote_path){}
static QoreValue FtpClient_put_VsNs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* local_path = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* remote_path = get_param_value(args, 1).get<const QoreStringNode>();
# 328 "QC_FtpClient.qpp"
    f->put(local_path->getBuffer(), remote_path ? remote_path->getBuffer() : 0, xsink);
    return QoreValue();
}

// nothing FtpClient::put(Qore::InputStream is, string remote_path){}
static QoreValue FtpClient_put_C11InputStreamVs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(is, InputStream, args, 0, CID_INPUTSTREAM, "FtpClient::put()", "Qore::InputStream", xsink);
    if (*xsink)
        return 0;
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 1);
# 353 "QC_FtpClient.qpp"
    SimpleRefHolder<InputStream> isHolder(is);
   f->put(is, remote_path->getBuffer(), xsink);
    return QoreValue();
}

// nothing FtpClient::putData(string data, string remote_path){}
static QoreValue FtpClient_putData_VsVs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* data = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 1);
# 448 "QC_FtpClient.qpp"
    f->putData(data->getBuffer(), data->size(), remote_path->getBuffer(), xsink);
    return QoreValue();
}

// nothing FtpClient::putData(binary data, string remote_path){}
static QoreValue FtpClient_putData_VyVs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* data = HARD_QORE_VALUE_BINARY(args, 0);
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 1);
# 469 "QC_FtpClient.qpp"
    f->putData(data->getPtr(), data->size(), remote_path->getBuffer(), xsink);
    return QoreValue();
}

// string FtpClient::pwd(){}
static QoreValue FtpClient_pwd(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 285 "QC_FtpClient.qpp"
    return f->pwd(xsink);
}

// nothing FtpClient::rename(string from, string to){}
static QoreValue FtpClient_rename_VsVs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* from = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* to = HARD_QORE_VALUE_STRING(args, 1);
# 375 "QC_FtpClient.qpp"
    f->rename(from->getBuffer(), to->getBuffer(), xsink);
    return QoreValue();
}

// nothing FtpClient::rmdir(string remote_path){}
static QoreValue FtpClient_rmdir_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* remote_path = HARD_QORE_VALUE_STRING(args, 0);
# 579 "QC_FtpClient.qpp"
    f->rmdir(remote_path->getBuffer(), xsink);
    return QoreValue();
}

// nothing FtpClient::setControlEventQueue(){}
static QoreValue FtpClient_setControlEventQueue(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 959 "QC_FtpClient.qpp"
    f->setControlEventQueue(0, xsink);
    return QoreValue();
}

// nothing FtpClient::setControlEventQueue(Qore::Thread::Queue queue){}
static QoreValue FtpClient_setControlEventQueue_C5Queue(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "FtpClient::setControlEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
# 975 "QC_FtpClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the FtpClient object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
   // pass reference from QoreObject::getReferencedPrivateData() to function
   f->setControlEventQueue(q.release(), xsink);
    return QoreValue();
}

// nothing FtpClient::setDataEventQueue(){}
static QoreValue FtpClient_setDataEventQueue(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 928 "QC_FtpClient.qpp"
    f->setDataEventQueue(0, xsink);
    return QoreValue();
}

// nothing FtpClient::setDataEventQueue(Qore::Thread::Queue queue){}
static QoreValue FtpClient_setDataEventQueue_C5Queue(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "FtpClient::setDataEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
# 944 "QC_FtpClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the FtpClient object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
   // pass reference from QoreObject::getReferencedPrivateData() to function
   f->setDataEventQueue(q.release(), xsink);
    return QoreValue();
}

// nothing FtpClient::setEventQueue(){}
static QoreValue FtpClient_setEventQueue(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 897 "QC_FtpClient.qpp"
    f->setEventQueue(0, xsink);
    return QoreValue();
}

// nothing FtpClient::setEventQueue(Qore::Thread::Queue queue){}
static QoreValue FtpClient_setEventQueue_C5Queue(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "FtpClient::setEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
# 913 "QC_FtpClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the FtpClient object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
   // pass reference from QoreObject::getReferencedPrivateData() to function
   f->setEventQueue(q.release(), xsink);
    return QoreValue();
}

// nothing FtpClient::setHostName(string host){}
static QoreValue FtpClient_setHostName_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* host = HARD_QORE_VALUE_STRING(args, 0);
# 616 "QC_FtpClient.qpp"
    f->setHostName(host->getBuffer());
    return QoreValue();
}

// nothing FtpClient::setInsecure(){}
static QoreValue FtpClient_setInsecure(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 746 "QC_FtpClient.qpp"
    if (f->setInsecure())
      xsink->raiseException("SET-INSECURE-ERROR", "this method cannot be called while the control connection is established");
    return QoreValue();
}

// nothing FtpClient::setInsecureData(){}
static QoreValue FtpClient_setInsecureData(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 761 "QC_FtpClient.qpp"
    if (f->setInsecureData())
      xsink->raiseException("SET-INSECUREDATA-ERROR", "this method cannot be called while the control connection is established");
    return QoreValue();
}

// nothing FtpClient::setModeAuto(){}
static QoreValue FtpClient_setModeAuto(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 841 "QC_FtpClient.qpp"
    f->setModeAuto();
    return QoreValue();
}

// nothing FtpClient::setModeEPSV(){}
static QoreValue FtpClient_setModeEPSV(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 852 "QC_FtpClient.qpp"
    f->setModeEPSV();
    return QoreValue();
}

// nothing FtpClient::setModePASV(){}
static QoreValue FtpClient_setModePASV(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 863 "QC_FtpClient.qpp"
    f->setModePASV();
    return QoreValue();
}

// nothing FtpClient::setModePORT(){}
static QoreValue FtpClient_setModePORT(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 874 "QC_FtpClient.qpp"
    f->setModePORT();
    return QoreValue();
}

// nothing FtpClient::setPassword(string pass){}
static QoreValue FtpClient_setPassword_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pass = HARD_QORE_VALUE_STRING(args, 0);
# 603 "QC_FtpClient.qpp"
    f->setPassword(pass->getBuffer());
    return QoreValue();
}

// nothing FtpClient::setPort(int port){}
static QoreValue FtpClient_setPort_Vi(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 port = HARD_QORE_VALUE_INT(args, 0);
# 630 "QC_FtpClient.qpp"
    if (port <= 0)
      xsink->raiseException("FTPCLIENT-SETPORT-PARAMETER-ERROR", "expecting positive port number as first parameter of FtpClient::setPort(softint port); got %d", port);
   else
      f->setPort(port);
    return QoreValue();
}

// nothing FtpClient::setSecure(bool secure = True){}
static QoreValue FtpClient_setSecure_Vb(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool secure = HARD_QORE_VALUE_BOOL(args, 0);
# 732 "QC_FtpClient.qpp"
    if (secure ? f->setSecure() : f->setInsecure())
      xsink->raiseException("SET-SECURE-ERROR", "this method cannot be called while the control connection is established");
    return QoreValue();
}

// nothing FtpClient::setTimeout(timeout timeout_ms){}
static QoreValue FtpClient_setTimeout_Vt(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1089 "QC_FtpClient.qpp"
    f->setTimeout(timeout_ms);
    return QoreValue();
}

// nothing FtpClient::setURL(string url){}
static QoreValue FtpClient_setURL_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* url = HARD_QORE_VALUE_STRING(args, 0);
# 650 "QC_FtpClient.qpp"
    f->setURL(url, xsink);
    return QoreValue();
}

// nothing FtpClient::setUserName(string user){}
static QoreValue FtpClient_setUserName_Vs(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* user = HARD_QORE_VALUE_STRING(args, 0);
# 591 "QC_FtpClient.qpp"
    f->setUserName(user->getBuffer());
    return QoreValue();
}

// nothing FtpClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
static QoreValue FtpClient_setWarningQueue_ViViC5QueueVAVt(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 warning_ms = HARD_QORE_VALUE_INT(args, 0);
    int64 warning_bs = HARD_QORE_VALUE_INT(args, 1);
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 2, CID_QUEUE, "FtpClient::setWarningQueue()", "Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 3);
    int64 min_ms = HARD_QORE_VALUE_INT(args, 4);
# 1030 "QC_FtpClient.qpp"
    ReferenceHolder<Queue> q(queue, xsink);
   if (queue->getMax() != -1)
      return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the Socket object if used; use a Queue object with no maximum size for I/O monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");
   // pass reference from QoreObject::getReferencedPrivateData() to function
   f->setWarningQueue(xsink, warning_ms, warning_bs, q.release(), arg.getReferencedValue(), min_ms);
    return QoreValue();
}

// *string FtpClient::verifyPeerCertificate(){}
static QoreValue FtpClient_verifyPeerCertificate(QoreObject* self, QoreFtpClientClass* f, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 829 "QC_FtpClient.qpp"
    const char *c = getSSLCVCode(f->verifyPeerCertificate());
   return c ? new QoreStringNode(c) : 0;
}

DLLLOCAL void preinitFtpClientClass() {
    QC_FTPCLIENT = new QoreClass("FtpClient", QDOM_NETWORK);
    CID_FTPCLIENT = QC_FTPCLIENT->getID();
    QC_FTPCLIENT->setSystem();
}

DLLLOCAL QoreClass* initFtpClientClass(QoreNamespace& ns) {
    if (!QC_FTPCLIENT)
        preinitFtpClientClass();

    // nothing FtpClient::clearStats(){}
    QC_FTPCLIENT->addMethod("clearStats", (q_method_n_t)FtpClient_clearStats, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::clearWarningQueue(){}
    QC_FTPCLIENT->addMethod("clearWarningQueue", (q_method_n_t)FtpClient_clearWarningQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::connect(){}
    QC_FTPCLIENT->addMethod("connect", (q_method_n_t)FtpClient_connect, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // FtpClient::constructor() {}
    QC_FTPCLIENT->addConstructor(FtpClient_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

    // FtpClient::constructor(string url) {}
    QC_FTPCLIENT->addConstructor(FtpClient_constructor_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, NULL, "url");

    // FtpClient::copy() {}
    QC_FTPCLIENT->setCopy((q_copy_t)FtpClient_copy);

    // nothing FtpClient::cwd(string path){}
    QC_FTPCLIENT->addMethod("cwd", (q_method_n_t)FtpClient_cwd_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "path");

    // nothing FtpClient::del(string remote_path){}
    QC_FTPCLIENT->addMethod("del", (q_method_n_t)FtpClient_del_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "remote_path");

    // FtpClient::destructor() {}
    QC_FTPCLIENT->setDestructor((q_destructor_t)FtpClient_destructor);

    // nothing FtpClient::disconnect(){}
    QC_FTPCLIENT->addMethod("disconnect", (q_method_n_t)FtpClient_disconnect, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::get(string remote_path, *string local_path){}
    QC_FTPCLIENT->addMethod("get", (q_method_n_t)FtpClient_get_VsNs, false, QC_NO_FLAGS, QDOM_FILESYSTEM, nothingTypeInfo, 2, stringTypeInfo, NULL, "remote_path", stringOrNothingTypeInfo, NULL, "local_path");

    // nothing FtpClient::get(string remote_path, Qore::OutputStream os){}
    QC_FTPCLIENT->addMethod("get", (q_method_n_t)FtpClient_get_VsC12OutputStream, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "remote_path", QC_OUTPUTSTREAM->getTypeInfo(), NULL, "os");

    // binary FtpClient::getAsBinary(string remote_path){}
    QC_FTPCLIENT->addMethod("getAsBinary", (q_method_n_t)FtpClient_getAsBinary_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, binaryTypeInfo, 1, stringTypeInfo, NULL, "remote_path");

    // string FtpClient::getAsString(string remote_path){}
    QC_FTPCLIENT->addMethod("getAsString", (q_method_n_t)FtpClient_getAsString_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, stringTypeInfo, NULL, "remote_path");

    // *string FtpClient::getHostName(){}
    QC_FTPCLIENT->addMethod("getHostName", (q_method_n_t)FtpClient_getHostName, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string FtpClient::getMode(){}
    QC_FTPCLIENT->addMethod("getMode", (q_method_n_t)FtpClient_getMode, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string FtpClient::getPassword(){}
    QC_FTPCLIENT->addMethod("getPassword", (q_method_n_t)FtpClient_getPassword, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int FtpClient::getPort(){}
    QC_FTPCLIENT->addMethod("getPort", (q_method_n_t)FtpClient_getPort, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // *string FtpClient::getSSLCipherName(){}
    QC_FTPCLIENT->addMethod("getSSLCipherName", (q_method_n_t)FtpClient_getSSLCipherName, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string FtpClient::getSSLCipherVersion(){}
    QC_FTPCLIENT->addMethod("getSSLCipherVersion", (q_method_n_t)FtpClient_getSSLCipherVersion, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // int FtpClient::getTimeout(){}
    QC_FTPCLIENT->addMethod("getTimeout", (q_method_n_t)FtpClient_getTimeout, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // string FtpClient::getURL(){}
    QC_FTPCLIENT->addMethod("getURL", (q_method_n_t)FtpClient_getURL, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // hash FtpClient::getUsageInfo(){}
    QC_FTPCLIENT->addMethod("getUsageInfo", (q_method_n_t)FtpClient_getUsageInfo, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // *string FtpClient::getUserName(){}
    QC_FTPCLIENT->addMethod("getUserName", (q_method_n_t)FtpClient_getUserName, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // bool FtpClient::isDataSecure(){}
    QC_FTPCLIENT->addMethod("isDataSecure", (q_method_n_t)FtpClient_isDataSecure, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool FtpClient::isSecure(){}
    QC_FTPCLIENT->addMethod("isSecure", (q_method_n_t)FtpClient_isSecure, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // *string FtpClient::list(){}
    QC_FTPCLIENT->addMethod("list", (q_method_n_t)FtpClient_list, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string FtpClient::list(string path){}
    QC_FTPCLIENT->addMethod("list", (q_method_n_t)FtpClient_list_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, stringTypeInfo, NULL, "path");

    // nothing FtpClient::mkdir(string remote_path){}
    QC_FTPCLIENT->addMethod("mkdir", (q_method_n_t)FtpClient_mkdir_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "remote_path");

    // *string FtpClient::nlst(){}
    QC_FTPCLIENT->addMethod("nlst", (q_method_n_t)FtpClient_nlst, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string FtpClient::nlst(string path){}
    QC_FTPCLIENT->addMethod("nlst", (q_method_n_t)FtpClient_nlst_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo, 1, stringTypeInfo, NULL, "path");

    // nothing FtpClient::put(string local_path, *string remote_path){}
    QC_FTPCLIENT->addMethod("put", (q_method_n_t)FtpClient_put_VsNs, false, QC_NO_FLAGS, QDOM_FILESYSTEM, nothingTypeInfo, 2, stringTypeInfo, NULL, "local_path", stringOrNothingTypeInfo, NULL, "remote_path");

    // nothing FtpClient::put(Qore::InputStream is, string remote_path){}
    QC_FTPCLIENT->addMethod("put", (q_method_n_t)FtpClient_put_C11InputStreamVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, QC_INPUTSTREAM->getTypeInfo(), NULL, "is", stringTypeInfo, NULL, "remote_path");

    // nothing FtpClient::putData(string data, string remote_path){}
    QC_FTPCLIENT->addMethod("putData", (q_method_n_t)FtpClient_putData_VsVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "data", stringTypeInfo, NULL, "remote_path");

    // nothing FtpClient::putData(binary data, string remote_path){}
    QC_FTPCLIENT->addMethod("putData", (q_method_n_t)FtpClient_putData_VyVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, binaryTypeInfo, NULL, "data", stringTypeInfo, NULL, "remote_path");

    // string FtpClient::pwd(){}
    QC_FTPCLIENT->addMethod("pwd", (q_method_n_t)FtpClient_pwd, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // nothing FtpClient::rename(string from, string to){}
    QC_FTPCLIENT->addMethod("rename", (q_method_n_t)FtpClient_rename_VsVs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "from", stringTypeInfo, NULL, "to");

    // nothing FtpClient::rmdir(string remote_path){}
    QC_FTPCLIENT->addMethod("rmdir", (q_method_n_t)FtpClient_rmdir_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "remote_path");

    // nothing FtpClient::setControlEventQueue(){}
    QC_FTPCLIENT->addMethod("setControlEventQueue", (q_method_n_t)FtpClient_setControlEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setControlEventQueue(Qore::Thread::Queue queue){}
    QC_FTPCLIENT->addMethod("setControlEventQueue", (q_method_n_t)FtpClient_setControlEventQueue_C5Queue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_QUEUE->getTypeInfo(), NULL, "queue");

    // nothing FtpClient::setDataEventQueue(){}
    QC_FTPCLIENT->addMethod("setDataEventQueue", (q_method_n_t)FtpClient_setDataEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setDataEventQueue(Qore::Thread::Queue queue){}
    QC_FTPCLIENT->addMethod("setDataEventQueue", (q_method_n_t)FtpClient_setDataEventQueue_C5Queue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_QUEUE->getTypeInfo(), NULL, "queue");

    // nothing FtpClient::setEventQueue(){}
    QC_FTPCLIENT->addMethod("setEventQueue", (q_method_n_t)FtpClient_setEventQueue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setEventQueue(Qore::Thread::Queue queue){}
    QC_FTPCLIENT->addMethod("setEventQueue", (q_method_n_t)FtpClient_setEventQueue_C5Queue, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_QUEUE->getTypeInfo(), NULL, "queue");

    // nothing FtpClient::setHostName(string host){}
    QC_FTPCLIENT->addMethod("setHostName", (q_method_n_t)FtpClient_setHostName_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "host");

    // nothing FtpClient::setInsecure(){}
    QC_FTPCLIENT->addMethod("setInsecure", (q_method_n_t)FtpClient_setInsecure, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setInsecureData(){}
    QC_FTPCLIENT->addMethod("setInsecureData", (q_method_n_t)FtpClient_setInsecureData, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setModeAuto(){}
    QC_FTPCLIENT->addMethod("setModeAuto", (q_method_n_t)FtpClient_setModeAuto, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setModeEPSV(){}
    QC_FTPCLIENT->addMethod("setModeEPSV", (q_method_n_t)FtpClient_setModeEPSV, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setModePASV(){}
    QC_FTPCLIENT->addMethod("setModePASV", (q_method_n_t)FtpClient_setModePASV, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setModePORT(){}
    QC_FTPCLIENT->addMethod("setModePORT", (q_method_n_t)FtpClient_setModePORT, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing FtpClient::setPassword(string pass){}
    QC_FTPCLIENT->addMethod("setPassword", (q_method_n_t)FtpClient_setPassword_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "pass");

    // nothing FtpClient::setPort(int port){}
    QC_FTPCLIENT->addMethod("setPort", (q_method_n_t)FtpClient_setPort_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, bigIntTypeInfo, NULL, "port");

    // nothing FtpClient::setSecure(bool secure = True){}
    QC_FTPCLIENT->addMethod("setSecure", (q_method_n_t)FtpClient_setSecure_Vb, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, boolTypeInfo, &True, "secure");

    // nothing FtpClient::setTimeout(timeout timeout_ms){}
    QC_FTPCLIENT->addMethod("setTimeout", (q_method_n_t)FtpClient_setTimeout_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, NULL, "timeout_ms");

    // nothing FtpClient::setURL(string url){}
    QC_FTPCLIENT->addMethod("setURL", (q_method_n_t)FtpClient_setURL_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "url");

    // nothing FtpClient::setUserName(string user){}
    QC_FTPCLIENT->addMethod("setUserName", (q_method_n_t)FtpClient_setUserName_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "user");

    // nothing FtpClient::setWarningQueue(int warning_ms, int warning_bs, Queue queue, auto arg, timeout min_ms = 1s){}
    QC_FTPCLIENT->addMethod("setWarningQueue", (q_method_n_t)FtpClient_setWarningQueue_ViViC5QueueVAVt, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 5, bigIntTypeInfo, NULL, "warning_ms", bigIntTypeInfo, NULL, "warning_bs", QC_QUEUE->getTypeInfo(), NULL, "queue", autoTypeInfo, NULL, "arg", timeoutTypeInfo, DateTimeNode::makeRelative(0, 0, 0, 0, 0, 1, 0), "min_ms");

    // *string FtpClient::verifyPeerCertificate(){}
    QC_FTPCLIENT->addMethod("verifyPeerCertificate", (q_method_n_t)FtpClient_verifyPeerCertificate, false, QC_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    return QC_FTPCLIENT;
}
