/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_HashListIterator.qpp HashListIterator class definition */
/*
  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreHashListIterator.h"
/* Qore class Qore::HashListIterator */

qore_classid_t CID_HASHLISTITERATOR;
QoreClass* QC_HASHLISTITERATOR;

// HashListIterator::constructor(hash h) {}
static void HashListIterator_constructor_Vh(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* h = HARD_QORE_VALUE_HASH(args, 0);
# 82 "QC_HashListIterator.qpp"
self->setPrivate(CID_HASHLISTITERATOR, new QoreHashListIterator(h));
}

// HashListIterator::constructor() {}
static void HashListIterator_constructor(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 93 "QC_HashListIterator.qpp"
self->setPrivate(CID_HASHLISTITERATOR, new QoreHashListIterator);
}

// HashListIterator::copy() {}
static void HashListIterator_copy(QoreObject* self, QoreObject* old, QoreHashListIterator* i, ExceptionSink* xsink) {
# 103 "QC_HashListIterator.qpp"
self->setPrivate(CID_HASHLISTITERATOR, new QoreHashListIterator(*i));
}

// bool HashListIterator::empty(){}
static QoreValue HashListIterator_empty(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 158 "QC_HashListIterator.qpp"
    return i->empty();
}

// bool HashListIterator::first(){}
static QoreValue HashListIterator_first(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 173 "QC_HashListIterator.qpp"
    return i->first();
}

// auto HashListIterator::getKeyValue(string key){}
static QoreValue HashListIterator_getKeyValue_Vs(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 327 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return QoreValue();

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// hash HashListIterator::getRow(){}
static QoreValue HashListIterator_getRow(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 231 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getRow(xsink);
}

// hash HashListIterator::getValue(){}
static QoreValue HashListIterator_getValue(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 209 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getRow(xsink);
}

// int HashListIterator::index(){}
static QoreValue HashListIterator_index(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 247 "QC_HashListIterator.qpp"
    return i->index();
}

// bool HashListIterator::last(){}
static QoreValue HashListIterator_last(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 188 "QC_HashListIterator.qpp"
    return i->last();
}

// int HashListIterator::max(){}
static QoreValue HashListIterator_max(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 261 "QC_HashListIterator.qpp"
    return i->max();
}

// auto HashListIterator::memberGate(string key){}
static QoreValue HashListIterator_memberGate_Vs(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 302 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return QoreValue();

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// bool HashListIterator::next(){}
static QoreValue HashListIterator_next(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 122 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->next();
}

// bool HashListIterator::prev(){}
static QoreValue HashListIterator_prev(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 143 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->prev();
}

// nothing HashListIterator::reset(){}
static QoreValue HashListIterator_reset(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 357 "QC_HashListIterator.qpp"
    if (i->check(xsink))
       return QoreValue();
   i->reset();
    return QoreValue();
}

// bool HashListIterator::set(int pos){}
static QoreValue HashListIterator_set_Vi(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 pos = HARD_QORE_VALUE_INT(args, 0);
# 278 "QC_HashListIterator.qpp"
    if (i->check(xsink))
      return false;
   return !i->set(pos);
}

// bool HashListIterator::valid(){}
static QoreValue HashListIterator_valid(QoreObject* self, QoreHashListIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 343 "QC_HashListIterator.qpp"
    return i->valid();
}

DLLLOCAL void preinitHashListIteratorClass() {
    QC_HASHLISTITERATOR = new QoreClass("HashListIterator", QDOM_DEFAULT);
    CID_HASHLISTITERATOR = QC_HASHLISTITERATOR->getID();
    QC_HASHLISTITERATOR->setSystem();
}

DLLLOCAL QoreClass* initHashListIteratorClass(QoreNamespace& ns) {
    if (!QC_HASHLISTITERATOR)
        preinitHashListIteratorClass();

    // set parent class
    assert(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);
    QC_HASHLISTITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);

    // HashListIterator::constructor(hash h) {}
    QC_HASHLISTITERATOR->addConstructor(HashListIterator_constructor_Vh, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, hashTypeInfo, NULL, "h");

    // HashListIterator::constructor() {}
    QC_HASHLISTITERATOR->addConstructor(HashListIterator_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

    // HashListIterator::copy() {}
    QC_HASHLISTITERATOR->setCopy((q_copy_t)HashListIterator_copy);

    // bool HashListIterator::empty(){}
    QC_HASHLISTITERATOR->addMethod("empty", (q_method_n_t)HashListIterator_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool HashListIterator::first(){}
    QC_HASHLISTITERATOR->addMethod("first", (q_method_n_t)HashListIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // auto HashListIterator::getKeyValue(string key){}
    QC_HASHLISTITERATOR->addMethod("getKeyValue", (q_method_n_t)HashListIterator_getKeyValue_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "key");

    // hash HashListIterator::getRow(){}
    QC_HASHLISTITERATOR->addMethod("getRow", (q_method_n_t)HashListIterator_getRow, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

    // hash HashListIterator::getValue(){}
    QC_HASHLISTITERATOR->addMethod("getValue", (q_method_n_t)HashListIterator_getValue, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

    // int HashListIterator::index(){}
    QC_HASHLISTITERATOR->addMethod("index", (q_method_n_t)HashListIterator_index, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool HashListIterator::last(){}
    QC_HASHLISTITERATOR->addMethod("last", (q_method_n_t)HashListIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int HashListIterator::max(){}
    QC_HASHLISTITERATOR->addMethod("max", (q_method_n_t)HashListIterator_max, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // auto HashListIterator::memberGate(string key){}
    QC_HASHLISTITERATOR->addMethod("memberGate", (q_method_n_t)HashListIterator_memberGate_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "key");

    // bool HashListIterator::next(){}
    QC_HASHLISTITERATOR->addMethod("next", (q_method_n_t)HashListIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool HashListIterator::prev(){}
    QC_HASHLISTITERATOR->addMethod("prev", (q_method_n_t)HashListIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing HashListIterator::reset(){}
    QC_HASHLISTITERATOR->addMethod("reset", (q_method_n_t)HashListIterator_reset, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool HashListIterator::set(int pos){}
    QC_HASHLISTITERATOR->addMethod("set", (q_method_n_t)HashListIterator_set_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, bigIntTypeInfo, NULL, "pos");

    // bool HashListIterator::valid(){}
    QC_HASHLISTITERATOR->addMethod("valid", (q_method_n_t)HashListIterator_valid, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_HASHLISTITERATOR;
}
