/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_ListHashIterator.qpp ListHashIterator class definition */
/*
  Qore Programming Language

  Copyright (C) 2003 - 2015 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QoreListHashIterator.h"
/* Qore class Qore::ListHashIterator */

qore_classid_t CID_LISTHASHITERATOR;
QoreClass* QC_LISTHASHITERATOR;

// ListHashIterator::constructor(softlist l) {}
static void ListHashIterator_constructor_vl(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 74 "QC_ListHashIterator.qpp"
self->setPrivate(CID_LISTHASHITERATOR, new QoreListHashIterator(l));
}

// ListHashIterator::copy() {}
static void ListHashIterator_copy(QoreObject* self, QoreObject* old, QoreListHashIterator* i, ExceptionSink* xsink) {
# 84 "QC_ListHashIterator.qpp"
self->setPrivate(CID_LISTHASHITERATOR, new QoreListHashIterator(*i));
}

// bool ListHashIterator::empty(){}
static QoreValue ListHashIterator_empty(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 139 "QC_ListHashIterator.qpp"
    return i->empty();
}

// bool ListHashIterator::first(){}
static QoreValue ListHashIterator_first(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 154 "QC_ListHashIterator.qpp"
    return i->first();
}

// auto ListHashIterator::getKeyValue(string key){}
static QoreValue ListHashIterator_getKeyValue_Vs(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 304 "QC_ListHashIterator.qpp"
    if (i->check(xsink))
      return QoreValue();

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// hash ListHashIterator::getRow(){}
static QoreValue ListHashIterator_getRow(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 210 "QC_ListHashIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getRow(xsink);
}

// hash ListHashIterator::getValue(){}
static QoreValue ListHashIterator_getValue(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 189 "QC_ListHashIterator.qpp"
    if (i->check(xsink))
      return QoreValue();
   return i->getRow(xsink);
}

// int ListHashIterator::index(){}
static QoreValue ListHashIterator_index(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 226 "QC_ListHashIterator.qpp"
    return i->index();
}

// bool ListHashIterator::last(){}
static QoreValue ListHashIterator_last(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 169 "QC_ListHashIterator.qpp"
    return i->last();
}

// int ListHashIterator::max(){}
static QoreValue ListHashIterator_max(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 240 "QC_ListHashIterator.qpp"
    return i->max();
}

// auto ListHashIterator::memberGate(string key){}
static QoreValue ListHashIterator_memberGate_Vs(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 280 "QC_ListHashIterator.qpp"
    if (i->check(xsink))
      return QoreValue();

   return i->getReferencedKeyValue(key->getBuffer(), xsink);
}

// bool ListHashIterator::next(){}
static QoreValue ListHashIterator_next(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 103 "QC_ListHashIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->next();
}

// bool ListHashIterator::prev(){}
static QoreValue ListHashIterator_prev(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 124 "QC_ListHashIterator.qpp"
    if (i->check(xsink))
      return false;
   return i->prev();
}

// nothing ListHashIterator::reset(){}
static QoreValue ListHashIterator_reset(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 334 "QC_ListHashIterator.qpp"
    if (i->check(xsink))
       return QoreValue();
   i->reset();
    return QoreValue();
}

// bool ListHashIterator::set(int pos){}
static QoreValue ListHashIterator_set_Vi(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 pos = HARD_QORE_VALUE_INT(args, 0);
# 257 "QC_ListHashIterator.qpp"
    if (i->check(xsink))
      return false;
   return !i->set(pos);
}

// bool ListHashIterator::valid(){}
static QoreValue ListHashIterator_valid(QoreObject* self, QoreListHashIterator* i, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 320 "QC_ListHashIterator.qpp"
    return i->valid();
}

DLLLOCAL void preinitListHashIteratorClass() {
    QC_LISTHASHITERATOR = new QoreClass("ListHashIterator", QDOM_DEFAULT);
    CID_LISTHASHITERATOR = QC_LISTHASHITERATOR->getID();
    QC_LISTHASHITERATOR->setSystem();
}

DLLLOCAL QoreClass* initListHashIteratorClass(QoreNamespace& ns) {
    if (!QC_LISTHASHITERATOR)
        preinitListHashIteratorClass();

    // set parent class
    assert(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);
    QC_LISTHASHITERATOR->addBuiltinVirtualBaseClass(QC_ABSTRACTQUANTIFIEDBIDIRECTIONALITERATOR);

    // ListHashIterator::constructor(softlist l) {}
    QC_LISTHASHITERATOR->addConstructor(ListHashIterator_constructor_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, softListTypeInfo, NULL, "l");

    // ListHashIterator::copy() {}
    QC_LISTHASHITERATOR->setCopy((q_copy_t)ListHashIterator_copy);

    // bool ListHashIterator::empty(){}
    QC_LISTHASHITERATOR->addMethod("empty", (q_method_n_t)ListHashIterator_empty, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool ListHashIterator::first(){}
    QC_LISTHASHITERATOR->addMethod("first", (q_method_n_t)ListHashIterator_first, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // auto ListHashIterator::getKeyValue(string key){}
    QC_LISTHASHITERATOR->addMethod("getKeyValue", (q_method_n_t)ListHashIterator_getKeyValue_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "key");

    // hash ListHashIterator::getRow(){}
    QC_LISTHASHITERATOR->addMethod("getRow", (q_method_n_t)ListHashIterator_getRow, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

    // hash ListHashIterator::getValue(){}
    QC_LISTHASHITERATOR->addMethod("getValue", (q_method_n_t)ListHashIterator_getValue, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, hashTypeInfo);

    // int ListHashIterator::index(){}
    QC_LISTHASHITERATOR->addMethod("index", (q_method_n_t)ListHashIterator_index, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool ListHashIterator::last(){}
    QC_LISTHASHITERATOR->addMethod("last", (q_method_n_t)ListHashIterator_last, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int ListHashIterator::max(){}
    QC_LISTHASHITERATOR->addMethod("max", (q_method_n_t)ListHashIterator_max, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // auto ListHashIterator::memberGate(string key){}
    QC_LISTHASHITERATOR->addMethod("memberGate", (q_method_n_t)ListHashIterator_memberGate_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "key");

    // bool ListHashIterator::next(){}
    QC_LISTHASHITERATOR->addMethod("next", (q_method_n_t)ListHashIterator_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // bool ListHashIterator::prev(){}
    QC_LISTHASHITERATOR->addMethod("prev", (q_method_n_t)ListHashIterator_prev, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing ListHashIterator::reset(){}
    QC_LISTHASHITERATOR->addMethod("reset", (q_method_n_t)ListHashIterator_reset, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool ListHashIterator::set(int pos){}
    QC_LISTHASHITERATOR->addMethod("set", (q_method_n_t)ListHashIterator_set_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, bigIntTypeInfo, NULL, "pos");

    // bool ListHashIterator::valid(){}
    QC_LISTHASHITERATOR->addMethod("valid", (q_method_n_t)ListHashIterator_valid, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_LISTHASHITERATOR;
}
