/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_ProgramControl.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_ProgramControl.h"
#include "qore/intern/QC_Breakpoint.h"
#include "qore/intern/QC_TimeZone.h"
#include "qore/intern/qore_program_private.h"
#include "qore/intern/QoreObjectIntern.h"
#include "qore/intern/qore_date_private.h"
#include "qore/intern/QoreHashNodeIntern.h"
TypedHashDecl* init_hashdecl_StatementInfo(QoreNamespace& ns) {
    TypedHashDecl* hd = new TypedHashDecl("StatementInfo");
    hd->addMember("end_line", bigIntTypeInfo, QoreValue());
    hd->addMember("file", stringTypeInfo, QoreValue());
    hd->addMember("offset", bigIntTypeInfo, QoreValue());
    hd->addMember("source", stringTypeInfo, QoreValue());
    hd->addMember("start_line", bigIntTypeInfo, QoreValue());
    ns.addSystemHashDecl(hd);
    return hd;
}

/* Qore class Qore::ProgramControl */

qore_classid_t CID_PROGRAMCONTROL;
QoreClass* QC_PROGRAMCONTROL;

// nothing ProgramControl::assignBreakpoint(Breakpoint bkpt){}
static QoreValue ProgramControl_assignBreakpoint_C10Breakpoint(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(bkpt, QoreBreakpoint, args, 0, CID_BREAKPOINT, "ProgramControl::assignBreakpoint()", "Breakpoint", xsink);
    if (*xsink)
        return 0;
# 265 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
      return QoreValue();
   p->assignBreakpoint(bkpt, xsink);
    return QoreValue();
}

// ProgramControl::constructor() {}
static void ProgramControl_constructor(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 81 "QC_ProgramControl.qpp"
xsink->raiseException("PROGRAM-COPY-ERROR", "creating ProgramControl objects is currently unsupported");
}

// ProgramControl::copy() {}
static void ProgramControl_copy(QoreObject* self, QoreObject* old, QoreProgram* p, ExceptionSink* xsink) {
# 89 "QC_ProgramControl.qpp"
xsink->raiseException("PROGRAM-COPY-ERROR", "copying ProgramControl objects is currently unsupported");
}

// nothing ProgramControl::deleteAllBreakpoints(){}
static QoreValue ProgramControl_deleteAllBreakpoints(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 274 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
      return QoreValue();
   p->deleteAllBreakpoints();
    return QoreValue();
}

// ProgramControl::destructor() {}
static void ProgramControl_destructor(QoreObject* self, QoreProgram* p, ExceptionSink* xsink) {
# 96 "QC_ProgramControl.qpp"
printd(5, "ProgramControl::constructor() pgm: %p, pgmid: %d, self: %p\n", p, p->getProgramId(), self);
   p->unregisterQoreObject(self, xsink);
   // the underlying object is created from C++ then it has reference and cannot be terminated
   p->deref(xsink);
}

// bool ProgramControl::existsFunction(string name){}
static QoreValue ProgramControl_existsFunction_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
# 210 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
      return false;
   TempEncodingHelper tmp(name, QCS_DEFAULT, xsink);
   if (!tmp)
      return false;
   return p->existsFunction(tmp->getBuffer());
}

// int ProgramControl::findFunctionStatementId(string function, *list params){}
static QoreValue ProgramControl_findFunctionStatementId_VsNl(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* function = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* params = get_param_value(args, 1).get<const QoreListNode>();
# 349 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
      return 0;
   ReferenceHolder<QoreValueList> vl(new QoreValueList(params), xsink);
   AbstractStatement* s = p->findFunctionStatement(function->c_str(), *vl, xsink);
   if (!s)
      return xsink->raiseException("PROGRAM-STATEMENT-ERROR", "cannot find statement for function \"%s\"", function->c_str());
   return p->getStatementId(s);
}

// list ProgramControl::findFunctionVariants(string function){}
static QoreValue ProgramControl_findFunctionVariants_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* function = HARD_QORE_VALUE_STRING(args, 0);
# 367 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
      return QoreValue();
   ReferenceHolder<QoreValueList> vl(p->runtimeFindCallVariants(function->c_str(), xsink), xsink);
   if (!vl)
      return xsink->raiseException("PROGRAM-ERROR", "cannot find any function \"%s\"", function->c_str());
   return vl->getOldList();
}

// int ProgramControl::findStatementId(string file, int line){}
static QoreValue ProgramControl_findStatementId_VsVi(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* file = HARD_QORE_VALUE_STRING(args, 0);
    int64 line = HARD_QORE_VALUE_INT(args, 1);
# 330 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
      return 0;
   AbstractStatement* s = p->findStatement(file->c_str(), line);
   if (!s) {
      return xsink->raiseException("PROGRAM-STATEMENT-ERROR", "cannot find statement for \"%s:%d\"", file->c_str(), line);
   }
   return p->getStatementId(s);
}

// *hash ProgramControl::getAllDefines(){}
static QoreValue ProgramControl_getAllDefines(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 157 "QC_ProgramControl.qpp"
    return qore_program_private::runTimeGetAllDefines(p);
}

// list ProgramControl::getBreakpoints(){}
static QoreValue ProgramControl_getBreakpoints(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 283 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
     return QoreValue();
   QoreBreakpointList_t bl;
   p->getBreakpoints(bl);
   QoreListNode *l = new QoreListNode;
   for (QoreBreakpointList_t::iterator it = bl.begin(); it != bl.end(); ++it) {
      l->push((*it)->getQoreObject());
   }
   return l;
}

// auto ProgramControl::getDefine(string def){}
static QoreValue ProgramControl_getDefine_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* def = HARD_QORE_VALUE_STRING(args, 0);
# 149 "QC_ProgramControl.qpp"
    TempEncodingHelper t(def, QCS_DEFAULT, xsink);
   return t ? qore_program_private::runTimeGetDefine(p, t->getBuffer()) : 0;
}

// auto ProgramControl::getGlobalVariable(string varname, *reference<bool> rexists){}
static QoreValue ProgramControl_getGlobalVariable_VsC16_reference_bool_(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* varname = HARD_QORE_VALUE_STRING(args, 0);
    const ReferenceNode* rexists = get_param_value(args, 1).get<const ReferenceNode>();
# 222 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
      return QoreValue();
   TempEncodingHelper t(varname, QCS_DEFAULT, xsink);
   if (!t)
      return QoreValue();

   bool found;
   ReferenceHolder<AbstractQoreNode> rv(p->getGlobalVariableValue(t->getBuffer(), found), xsink);

   if (rexists) {
     QoreTypeSafeReferenceHelper r(rexists, xsink);
     if (!r)
       return QoreValue();

     if (r.assign(get_bool_node(found), xsink))
       return QoreValue();
   }

   return rv.release();
}

// hash ProgramControl::getGlobalVars(){}
static QoreValue ProgramControl_getGlobalVars(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 247 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
      return QoreValue();
   return p->getGlobalVars();
}

// int ProgramControl::getParseOptions(){}
static QoreValue ProgramControl_getParseOptions(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 106 "QC_ProgramControl.qpp"
    return p->getParseOptions64();
}

// int ProgramControl::getProgramId(){}
static QoreValue ProgramControl_getProgramId(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 164 "QC_ProgramControl.qpp"
    return p->getProgramId();
}

// *string ProgramControl::getScriptDir(){}
static QoreValue ProgramControl_getScriptDir(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 113 "QC_ProgramControl.qpp"
    return p->getScriptDir();
}

// *string ProgramControl::getScriptName(){}
static QoreValue ProgramControl_getScriptName(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 120 "QC_ProgramControl.qpp"
    return p->getScriptName();
}

// *string ProgramControl::getScriptPath(){}
static QoreValue ProgramControl_getScriptPath(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 127 "QC_ProgramControl.qpp"
    return p->getScriptPath();
}

// hash<StatementInfo> ProgramControl::getStatementIdInfo(int statementId){}
static QoreValue ProgramControl_getStatementIdInfo_Vi(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 statementId = HARD_QORE_VALUE_INT(args, 0);
# 303 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
      return QoreValue();
   AbstractStatement* s = p->resolveStatementId(statementId);
   if (!s) {
      return xsink->raiseException("PROGRAM-STATEMENT-ERROR", "cannot resolve statement id from (%d)", statementId);
   }
   QoreHashNode* h = new QoreHashNode(hashdeclStatementInfo, nullptr);
   qore_hash_private* ph = qore_hash_private::get(*h);
   ph->setKeyValueIntern("file", new QoreStringNode(s->loc.file));
   ph->setKeyValueIntern("source", new QoreStringNode(s->loc.source));
   ph->setKeyValueIntern("offset", new QoreBigIntNode(s->loc.offset));
   ph->setKeyValueIntern("start_line", new QoreBigIntNode(s->loc.start_line));
   ph->setKeyValueIntern("end_line", new QoreBigIntNode(s->loc.end_line));
   return h;
}

// list<int> ProgramControl::getThreadList(){}
static QoreValue ProgramControl_getThreadList(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 203 "QC_ProgramControl.qpp"
    return p->getThreadList();
}

// TimeZone ProgramControl::getTimeZone(){}
static QoreValue ProgramControl_getTimeZone(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 134 "QC_ProgramControl.qpp"
    return new QoreObject(QC_TIMEZONE, 0, new TimeZoneData(p->currentTZ()));
}

// bool ProgramControl::isDefined(string def){}
static QoreValue ProgramControl_isDefined_Vs(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* def = HARD_QORE_VALUE_STRING(args, 0);
# 141 "QC_ProgramControl.qpp"
    TempEncodingHelper t(def, QCS_DEFAULT, xsink);
   return t ? qore_program_private::runTimeIsDefined(p, t->getBuffer()) : false;
}

// nothing ProgramControl::setGlobalVarValue(string name, auto value){}
static QoreValue ProgramControl_setGlobalVarValue_VsVA(QoreObject* self, QoreProgram* p, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* name = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue value = get_param_value(args, 1);
# 256 "QC_ProgramControl.qpp"
    if (!p->checkAllowDebugging(xsink))
      return QoreValue();
   p->setGlobalVarValue(name->c_str(), value.refSelf(), xsink);
    return QoreValue();
}

// static list ProgramControl::getAllPrograms(){}
static QoreValue static_ProgramControl_getAllPrograms(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 196 "QC_ProgramControl.qpp"
return QoreProgram::getAllQoreObjects(xsink);
}

// static ProgramControl ProgramControl::getProgram(){}
static QoreValue static_ProgramControl_getProgram(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 187 "QC_ProgramControl.qpp"
QoreProgram* pgm = getProgram();
   return QoreProgram::getQoreObject(pgm);
}

// static ProgramControl ProgramControl::resolveProgramId(int programId){}
static QoreValue static_ProgramControl_resolveProgramId_Vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 programId = HARD_QORE_VALUE_INT(args, 0);
# 176 "QC_ProgramControl.qpp"
QoreProgram* pgm = QoreProgram::resolveProgramId(programId);
   if (!pgm) {
      return xsink->raiseException("PROGRAM-ERROR", "cannot find program for programId (%d)", programId);
   }
   return QoreProgram::getQoreObject(pgm);
}

DLLLOCAL void preinitProgramControlClass() {
    QC_PROGRAMCONTROL = new QoreClass("ProgramControl", QDOM_DEBUGGER);
    CID_PROGRAMCONTROL = QC_PROGRAMCONTROL->getID();
    QC_PROGRAMCONTROL->setSystem();
}

DLLLOCAL QoreClass* initProgramControlClass(QoreNamespace& ns) {
    if (!QC_PROGRAMCONTROL)
        preinitProgramControlClass();

    // nothing ProgramControl::assignBreakpoint(Breakpoint bkpt){}
    QC_PROGRAMCONTROL->addMethod("assignBreakpoint", (q_method_n_t)ProgramControl_assignBreakpoint_C10Breakpoint, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, QC_BREAKPOINT->getTypeInfo(), NULL, "bkpt");

    // ProgramControl::constructor() {}
    QC_PROGRAMCONTROL->addConstructor(ProgramControl_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

    // ProgramControl::copy() {}
    QC_PROGRAMCONTROL->setCopy((q_copy_t)ProgramControl_copy);

    // nothing ProgramControl::deleteAllBreakpoints(){}
    QC_PROGRAMCONTROL->addMethod("deleteAllBreakpoints", (q_method_n_t)ProgramControl_deleteAllBreakpoints, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // ProgramControl::destructor() {}
    QC_PROGRAMCONTROL->setDestructor((q_destructor_t)ProgramControl_destructor);

    // bool ProgramControl::existsFunction(string name){}
    QC_PROGRAMCONTROL->addMethod("existsFunction", (q_method_n_t)ProgramControl_existsFunction_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "name");

    // int ProgramControl::findFunctionStatementId(string function, *list params){}
    QC_PROGRAMCONTROL->addMethod("findFunctionStatementId", (q_method_n_t)ProgramControl_findFunctionStatementId_VsNl, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, NULL, "function", listOrNothingTypeInfo, NULL, "params");

    // list ProgramControl::findFunctionVariants(string function){}
    QC_PROGRAMCONTROL->addMethod("findFunctionVariants", (q_method_n_t)ProgramControl_findFunctionVariants_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 1, stringTypeInfo, NULL, "function");

    // int ProgramControl::findStatementId(string file, int line){}
    QC_PROGRAMCONTROL->addMethod("findStatementId", (q_method_n_t)ProgramControl_findStatementId_VsVi, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 2, stringTypeInfo, NULL, "file", bigIntTypeInfo, NULL, "line");

    // *hash ProgramControl::getAllDefines(){}
    QC_PROGRAMCONTROL->addMethod("getAllDefines", (q_method_n_t)ProgramControl_getAllDefines, false, QC_NO_FLAGS, QDOM_DEFAULT, hashOrNothingTypeInfo);

    // list ProgramControl::getBreakpoints(){}
    QC_PROGRAMCONTROL->addMethod("getBreakpoints", (q_method_n_t)ProgramControl_getBreakpoints, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo);

    // auto ProgramControl::getDefine(string def){}
    QC_PROGRAMCONTROL->addMethod("getDefine", (q_method_n_t)ProgramControl_getDefine_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "def");

    // auto ProgramControl::getGlobalVariable(string varname, *reference<bool> rexists){}
    QC_PROGRAMCONTROL->addMethod("getGlobalVariable", (q_method_n_t)ProgramControl_getGlobalVariable_VsC16_reference_bool_, false, QC_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, NULL, "varname", qore_get_complex_reference_or_nothing_type(boolTypeInfo), NULL, "rexists");

    // hash ProgramControl::getGlobalVars(){}
    QC_PROGRAMCONTROL->addMethod("getGlobalVars", (q_method_n_t)ProgramControl_getGlobalVars, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // int ProgramControl::getParseOptions(){}
    QC_PROGRAMCONTROL->addMethod("getParseOptions", (q_method_n_t)ProgramControl_getParseOptions, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int ProgramControl::getProgramId(){}
    QC_PROGRAMCONTROL->addMethod("getProgramId", (q_method_n_t)ProgramControl_getProgramId, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // *string ProgramControl::getScriptDir(){}
    QC_PROGRAMCONTROL->addMethod("getScriptDir", (q_method_n_t)ProgramControl_getScriptDir, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string ProgramControl::getScriptName(){}
    QC_PROGRAMCONTROL->addMethod("getScriptName", (q_method_n_t)ProgramControl_getScriptName, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string ProgramControl::getScriptPath(){}
    QC_PROGRAMCONTROL->addMethod("getScriptPath", (q_method_n_t)ProgramControl_getScriptPath, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // hash<StatementInfo> ProgramControl::getStatementIdInfo(int statementId){}
    QC_PROGRAMCONTROL->addMethod("getStatementIdInfo", (q_method_n_t)ProgramControl_getStatementIdInfo_Vi, false, QC_NO_FLAGS, QDOM_DEFAULT, hashdeclStatementInfo->getTypeInfo(false), 1, bigIntTypeInfo, NULL, "statementId");

    // list<int> ProgramControl::getThreadList(){}
    QC_PROGRAMCONTROL->addMethod("getThreadList", (q_method_n_t)ProgramControl_getThreadList, false, QC_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(bigIntTypeInfo));

    // TimeZone ProgramControl::getTimeZone(){}
    QC_PROGRAMCONTROL->addMethod("getTimeZone", (q_method_n_t)ProgramControl_getTimeZone, false, QC_NO_FLAGS, QDOM_DEFAULT, QC_TIMEZONE->getTypeInfo());

    // bool ProgramControl::isDefined(string def){}
    QC_PROGRAMCONTROL->addMethod("isDefined", (q_method_n_t)ProgramControl_isDefined_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, NULL, "def");

    // nothing ProgramControl::setGlobalVarValue(string name, auto value){}
    QC_PROGRAMCONTROL->addMethod("setGlobalVarValue", (q_method_n_t)ProgramControl_setGlobalVarValue_VsVA, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, NULL, "name", autoTypeInfo, NULL, "value");

    // static list ProgramControl::getAllPrograms(){}
    QC_PROGRAMCONTROL->addStaticMethod("getAllPrograms", (q_func_n_t)static_ProgramControl_getAllPrograms, false, QC_RET_VALUE_ONLY, QDOM_DEBUGGER, listTypeInfo);

    // static ProgramControl ProgramControl::getProgram(){}
    QC_PROGRAMCONTROL->addStaticMethod("getProgram", (q_func_n_t)static_ProgramControl_getProgram, false, QC_RET_VALUE_ONLY, QDOM_DEBUGGER, QC_PROGRAMCONTROL->getTypeInfo());

    // static ProgramControl ProgramControl::resolveProgramId(int programId){}
    QC_PROGRAMCONTROL->addStaticMethod("resolveProgramId", (q_func_n_t)static_ProgramControl_resolveProgramId_Vi, false, QC_NO_FLAGS, QDOM_DEBUGGER, QC_PROGRAMCONTROL->getTypeInfo(), 1, bigIntTypeInfo, NULL, "programId");

    return QC_PROGRAMCONTROL;
}
