/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_RWLock.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2015 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_RWLock.h"
#include "qore/intern/QC_AbstractSmartLock.h"
/* Qore class Qore::Thread::RWLock */

qore_classid_t CID_RWLOCK;
QoreClass* QC_RWLOCK;

// RWLock::constructor() {}
static void RWLock_constructor(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 78 "QC_RWLock.qpp"
self->setPrivate(CID_RWLOCK, new RWLock);
}

// RWLock::copy() {}
static void RWLock_copy(QoreObject* self, QoreObject* old, RWLock* rwl, ExceptionSink* xsink) {
# 104 "QC_RWLock.qpp"
self->setPrivate(CID_RWLOCK, new RWLock);
}

// RWLock::destructor() {}
static void RWLock_destructor(QoreObject* self, RWLock* rwl, ExceptionSink* xsink) {
# 92 "QC_RWLock.qpp"
rwl->destructor(xsink);
   rwl->deref(xsink);
}

// int RWLock::getReadWaiting(){}
static QoreValue RWLock_getReadWaiting(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 252 "QC_RWLock.qpp"
    return rwl->getReadWaiting();
}

// int RWLock::getWriteWaiting(){}
static QoreValue RWLock_getWriteWaiting(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 264 "QC_RWLock.qpp"
    return rwl->getWriteWaiting();
}

// bool RWLock::lockOwner(){}
static QoreValue RWLock_lockOwner(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 277 "QC_RWLock.qpp"
    return rwl->lockOwner();
}

// int RWLock::numReaders(){}
static QoreValue RWLock_numReaders(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 240 "QC_RWLock.qpp"
    return rwl->numReaders();
}

// nothing RWLock::readLock(){}
static QoreValue RWLock_readLock(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 118 "QC_RWLock.qpp"
    rwl->readLock(xsink);
    return QoreValue();
}

// int RWLock::readLock(timeout timeout_ms){}
static QoreValue RWLock_readLock_Vt(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 138 "QC_RWLock.qpp"
    return rwl->readLock(xsink, timeout_ms);
}

// bool RWLock::readLockOwner(){}
static QoreValue RWLock_readLockOwner(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 290 "QC_RWLock.qpp"
    return rwl->readLockOwner();
}

// nothing RWLock::readUnlock(){}
static QoreValue RWLock_readUnlock(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 151 "QC_RWLock.qpp"
    rwl->readUnlock(xsink);
    return QoreValue();
}

// int RWLock::tryReadLock(){}
static QoreValue RWLock_tryReadLock(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 213 "QC_RWLock.qpp"
    return rwl->tryReadLock();
}

// int RWLock::tryWriteLock(){}
static QoreValue RWLock_tryWriteLock(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 227 "QC_RWLock.qpp"
    return rwl->tryGrab();
}

// int RWLock::writeLock(timeout timeout_ms){}
static QoreValue RWLock_writeLock_Vt(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 171 "QC_RWLock.qpp"
    return rwl->grab(xsink, timeout_ms);
}

// nothing RWLock::writeLock(){}
static QoreValue RWLock_writeLock(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 185 "QC_RWLock.qpp"
    rwl->grab(xsink);
    return QoreValue();
}

// bool RWLock::writeLockOwner(){}
static QoreValue RWLock_writeLockOwner(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 303 "QC_RWLock.qpp"
    return rwl->writeLockOwner();
}

// nothing RWLock::writeUnlock(){}
static QoreValue RWLock_writeUnlock(QoreObject* self, RWLock* rwl, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 198 "QC_RWLock.qpp"
    rwl->release(xsink);
    return QoreValue();
}

DLLLOCAL void preinitRWLockClass() {
    QC_RWLOCK = new QoreClass("RWLock", QDOM_THREAD_CLASS);
    CID_RWLOCK = QC_RWLOCK->getID();
    QC_RWLOCK->setSystem();
}

DLLLOCAL QoreClass* initRWLockClass(QoreNamespace& ns) {
    if (!QC_RWLOCK)
        preinitRWLockClass();

    // set parent class
    assert(QC_ABSTRACTSMARTLOCK);
    QC_RWLOCK->addBuiltinVirtualBaseClass(QC_ABSTRACTSMARTLOCK);

    // RWLock::constructor() {}
    QC_RWLOCK->addConstructor(RWLock_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

    // RWLock::copy() {}
    QC_RWLOCK->setCopy((q_copy_t)RWLock_copy);

    // RWLock::destructor() {}
    QC_RWLOCK->setDestructor((q_destructor_t)RWLock_destructor);

    // int RWLock::getReadWaiting(){}
    QC_RWLOCK->addMethod("getReadWaiting", (q_method_n_t)RWLock_getReadWaiting, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // int RWLock::getWriteWaiting(){}
    QC_RWLOCK->addMethod("getWriteWaiting", (q_method_n_t)RWLock_getWriteWaiting, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // bool RWLock::lockOwner(){}
    QC_RWLOCK->addMethod("lockOwner", (q_method_n_t)RWLock_lockOwner, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int RWLock::numReaders(){}
    QC_RWLOCK->addMethod("numReaders", (q_method_n_t)RWLock_numReaders, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing RWLock::readLock(){}
    QC_RWLOCK->addMethod("readLock", (q_method_n_t)RWLock_readLock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int RWLock::readLock(timeout timeout_ms){}
    QC_RWLOCK->addMethod("readLock", (q_method_n_t)RWLock_readLock_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, NULL, "timeout_ms");

    // bool RWLock::readLockOwner(){}
    QC_RWLOCK->addMethod("readLockOwner", (q_method_n_t)RWLock_readLockOwner, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // nothing RWLock::readUnlock(){}
    QC_RWLOCK->addMethod("readUnlock", (q_method_n_t)RWLock_readUnlock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // int RWLock::tryReadLock(){}
    QC_RWLOCK->addMethod("tryReadLock", (q_method_n_t)RWLock_tryReadLock, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int RWLock::tryWriteLock(){}
    QC_RWLOCK->addMethod("tryWriteLock", (q_method_n_t)RWLock_tryWriteLock, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int RWLock::writeLock(timeout timeout_ms){}
    QC_RWLOCK->addMethod("writeLock", (q_method_n_t)RWLock_writeLock_Vt, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, timeoutTypeInfo, NULL, "timeout_ms");

    // nothing RWLock::writeLock(){}
    QC_RWLOCK->addMethod("writeLock", (q_method_n_t)RWLock_writeLock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool RWLock::writeLockOwner(){}
    QC_RWLOCK->addMethod("writeLockOwner", (q_method_n_t)RWLock_writeLockOwner, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // nothing RWLock::writeUnlock(){}
    QC_RWLOCK->addMethod("writeUnlock", (q_method_n_t)RWLock_writeUnlock, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    return QC_RWLOCK;
}
