/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_SQLStatement.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_SQLStatement.h"
#include "qore/intern/QC_Datasource.h"
#include "qore/intern/QC_DatasourcePool.h"
/* Qore class Qore::SQL::SQLStatement */

qore_classid_t CID_SQLSTATEMENT;
QoreClass* QC_SQLSTATEMENT;

// bool SQLStatement::active(){}
static QoreValue SQLStatement_active(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 724 "QC_SQLStatement.qpp"
    return stmt->active();
}

// int SQLStatement::affectedRows(){}
static QoreValue SQLStatement_affectedRows(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 461 "QC_SQLStatement.qpp"
    return stmt->affectedRows(xsink);
}

// nothing SQLStatement::beginTransaction(){}
static QoreValue SQLStatement_beginTransaction(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 565 "QC_SQLStatement.qpp"
    stmt->beginTransaction(xsink);
    return QoreValue();
}

// nothing SQLStatement::bind(...){}
static QoreValue SQLStatement_bind_VV(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 229 "QC_SQLStatement.qpp"
    ReferenceHolder<QoreListNode> l(args ? args->getOldList() : 0, xsink);
   stmt->bind(**l, xsink);
    return QoreValue();
}

// nothing SQLStatement::bindArgs(softlist vargs){}
static QoreValue SQLStatement_bindArgs_vl(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* vargs = HARD_QORE_VALUE_LIST(args, 0);
# 261 "QC_SQLStatement.qpp"
    stmt->bind(*vargs, xsink);
    return QoreValue();
}

// nothing SQLStatement::bindPlaceholders(...){}
static QoreValue SQLStatement_bindPlaceholders_VV(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 290 "QC_SQLStatement.qpp"
    ReferenceHolder<QoreListNode> l(args ? args->getOldList() : 0, xsink);
   stmt->bindPlaceholders(**l, xsink);
    return QoreValue();
}

// nothing SQLStatement::bindPlaceholdersArgs(softlist vargs){}
static QoreValue SQLStatement_bindPlaceholdersArgs_vl(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* vargs = HARD_QORE_VALUE_LIST(args, 0);
# 321 "QC_SQLStatement.qpp"
    stmt->bindPlaceholders(*vargs, xsink);
    return QoreValue();
}

// nothing SQLStatement::bindValues(...){}
static QoreValue SQLStatement_bindValues_VV(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 351 "QC_SQLStatement.qpp"
    ReferenceHolder<QoreListNode> l(args ? args->getOldList() : 0, xsink);
   stmt->bindValues(**l, xsink);
    return QoreValue();
}

// nothing SQLStatement::bindValuesArgs(softlist vargs){}
static QoreValue SQLStatement_bindValuesArgs_vl(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* vargs = HARD_QORE_VALUE_LIST(args, 0);
# 381 "QC_SQLStatement.qpp"
    stmt->bindValues(*vargs, xsink);
    return QoreValue();
}

// nothing SQLStatement::close(){}
static QoreValue SQLStatement_close(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 528 "QC_SQLStatement.qpp"
    stmt->close(xsink);
    return QoreValue();
}

// nothing SQLStatement::commit(){}
static QoreValue SQLStatement_commit(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 541 "QC_SQLStatement.qpp"
    stmt->commit(xsink);
    return QoreValue();
}

// SQLStatement::constructor(Datasource ds) {}
static void SQLStatement_constructor_C10Datasource(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(ds, ManagedDatasource, args, 0, CID_DATASOURCE, "SQLStatement::constructor()", "Datasource", xsink);
    if (*xsink)
        return;
# 109 "QC_SQLStatement.qpp"
ReferenceHolder<ManagedDatasource> ds_holder(ds, xsink);

   if (!ds->getDriver()->hasStatementAPI()) {
      xsink->raiseException("SQLSTATEMENT-ERROR", "DBI driver '%s' does not support the prepared statement API", ds->getDriver()->getName());
      return;
   }

   self->setPrivate(CID_SQLSTATEMENT, new QoreSQLStatement(ds));
}

// SQLStatement::constructor(DatasourcePool dsp) {}
static void SQLStatement_constructor_C14DatasourcePool(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(dsp, DatasourcePool, args, 0, CID_DATASOURCEPOOL, "SQLStatement::constructor()", "DatasourcePool", xsink);
    if (*xsink)
        return;
# 134 "QC_SQLStatement.qpp"
ReferenceHolder<DatasourcePool> dsp_holder(dsp, xsink);

   if (!dsp->getDriver()->hasStatementAPI()) {
      xsink->raiseException("SQLSTATEMENT-ERROR", "DBI driver '%s' does not support the prepared statement API", dsp->getDriver()->getName());
      return;
   }

   self->setPrivate(CID_SQLSTATEMENT, new QoreSQLStatement(dsp));
}

// SQLStatement::copy() {}
static void SQLStatement_copy(QoreObject* self, QoreObject* old, QoreSQLStatement* stmt, ExceptionSink* xsink) {
# 161 "QC_SQLStatement.qpp"
xsink->raiseException("SQLSTATEMENT-COPY-ERROR", "SQLStatement objects cannot be copied");
   //self->setPrivate(CID_SQLSTATEMENT, new SQLStatement(*old));
}

// bool SQLStatement::currentThreadInTransaction(){}
static QoreValue SQLStatement_currentThreadInTransaction(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 781 "QC_SQLStatement.qpp"
    return stmt->currentThreadInTransaction(xsink);
}

// nothing SQLStatement::define(){}
static QoreValue SQLStatement_define(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 517 "QC_SQLStatement.qpp"
    stmt->define(xsink);
    return QoreValue();
}

// hash SQLStatement::describe(){}
static QoreValue SQLStatement_describe(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 699 "QC_SQLStatement.qpp"
    return stmt->describe(xsink);
}

// SQLStatement::destructor() {}
static void SQLStatement_destructor(QoreObject* self, QoreSQLStatement* stmt, ExceptionSink* xsink) {
# 152 "QC_SQLStatement.qpp"
stmt->close(xsink);
   stmt->deref(xsink);
}

// nothing SQLStatement::exec(...){}
static QoreValue SQLStatement_exec_VV(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 412 "QC_SQLStatement.qpp"
    ReferenceHolder<QoreListNode> l(args ? args->getOldList() : 0, xsink);
   stmt->exec(*l, xsink);
    return QoreValue();
}

// nothing SQLStatement::execArgs(softlist vargs){}
static QoreValue SQLStatement_execArgs_vl(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* vargs = HARD_QORE_VALUE_LIST(args, 0);
# 445 "QC_SQLStatement.qpp"
    stmt->exec(vargs, xsink);
    return QoreValue();
}

// hash SQLStatement::fetchColumns(softint rows = -1){}
static QoreValue SQLStatement_fetchColumns_vi(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 rows = HARD_QORE_VALUE_INT(args, 0);
# 686 "QC_SQLStatement.qpp"
    return stmt->fetchColumns((int)rows, xsink);
}

// *hash SQLStatement::fetchRow(){}
static QoreValue SQLStatement_fetchRow(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 618 "QC_SQLStatement.qpp"
    return stmt->fetchRow(xsink);
}

// list SQLStatement::fetchRows(softint rows = -1){}
static QoreValue SQLStatement_fetchRows_vi(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 rows = HARD_QORE_VALUE_INT(args, 0);
# 664 "QC_SQLStatement.qpp"
    return stmt->fetchRows((int)rows, xsink);
}

// hash SQLStatement::getOutput(){}
static QoreValue SQLStatement_getOutput(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 477 "QC_SQLStatement.qpp"
    return stmt->getOutput(xsink);
}

// hash SQLStatement::getOutputRows(){}
static QoreValue SQLStatement_getOutputRows(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 493 "QC_SQLStatement.qpp"
    return stmt->getOutputRows(xsink);
}

// *string SQLStatement::getSQL(){}
static QoreValue SQLStatement_getSQL(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 711 "QC_SQLStatement.qpp"
    return stmt->getSQL(xsink);
}

// *hash SQLStatement::getValue(){}
static QoreValue SQLStatement_getValue(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 642 "QC_SQLStatement.qpp"
    return stmt->fetchRow(xsink);
}

// auto SQLStatement::memberGate(string key){}
static QoreValue SQLStatement_memberGate_Vs(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* key = HARD_QORE_VALUE_STRING(args, 0);
# 753 "QC_SQLStatement.qpp"
    ReferenceHolder<QoreHashNode> h(stmt->fetchRow(xsink), xsink);
   if (*xsink)
      return QoreValue();
   if (!h)
      return xsink->raiseException("SQLSTATEMENT-ITERATION-ERROR", "Cannot dereference statement iteration context to get current '%s' column value; make sure and call SQLStatement::next() before trying to dereference the current row being iterated", key->getBuffer());

   bool exists = false;
   const AbstractQoreNode* rv = h->getKeyValueExistence(key, exists, xsink);
   if (*xsink)
      return QoreValue();

   if (!exists)
      return xsink->raiseException("SQLSTATEMENT-COLUMN-ERROR", "The given column name ('%s') does not exist in the current row data", key->getBuffer());

   return rv ? rv->refSelf() : QoreValue();
}

// bool SQLStatement::next(){}
static QoreValue SQLStatement_next(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 585 "QC_SQLStatement.qpp"
    return stmt->next(xsink);
}

// nothing SQLStatement::prepare(string sql, ...){}
static QoreValue SQLStatement_prepare_VsVV(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 178 "QC_SQLStatement.qpp"
    ReferenceHolder<QoreListNode> l(xsink);

   if (num_args(args) > 1)
      l = args->getOldList(1);

   stmt->prepare(*sql, *l, xsink);
    return QoreValue();
}

// nothing SQLStatement::prepareRaw(string sql){}
static QoreValue SQLStatement_prepareRaw_Vs(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 199 "QC_SQLStatement.qpp"
    stmt->prepareRaw(*sql, xsink);
    return QoreValue();
}

// nothing SQLStatement::rollback(){}
static QoreValue SQLStatement_rollback(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 554 "QC_SQLStatement.qpp"
    stmt->rollback(xsink);
    return QoreValue();
}

// bool SQLStatement::valid(){}
static QoreValue SQLStatement_valid(QoreObject* self, QoreSQLStatement* stmt, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 598 "QC_SQLStatement.qpp"
    return stmt->valid();
}

DLLLOCAL void preinitSQLStatementClass() {
    QC_SQLSTATEMENT = new QoreClass("SQLStatement", QDOM_DATABASE);
    CID_SQLSTATEMENT = QC_SQLSTATEMENT->getID();
    QC_SQLSTATEMENT->setSystem();
}

DLLLOCAL QoreClass* initSQLStatementClass(QoreNamespace& ns) {
    if (!QC_SQLSTATEMENT)
        preinitSQLStatementClass();

    // set parent class
    assert(QC_ABSTRACTITERATOR);
    QC_SQLSTATEMENT->addBuiltinVirtualBaseClass(QC_ABSTRACTITERATOR);

    // bool SQLStatement::active(){}
    QC_SQLSTATEMENT->addMethod("active", (q_method_n_t)SQLStatement_active, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int SQLStatement::affectedRows(){}
    QC_SQLSTATEMENT->addMethod("affectedRows", (q_method_n_t)SQLStatement_affectedRows, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // nothing SQLStatement::beginTransaction(){}
    QC_SQLSTATEMENT->addMethod("beginTransaction", (q_method_n_t)SQLStatement_beginTransaction, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing SQLStatement::bind(...){}
    QC_SQLSTATEMENT->addMethod("bind", (q_method_n_t)SQLStatement_bind_VV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing SQLStatement::bindArgs(softlist vargs){}
    QC_SQLSTATEMENT->addMethod("bindArgs", (q_method_n_t)SQLStatement_bindArgs_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softListTypeInfo, NULL, "vargs");

    // nothing SQLStatement::bindPlaceholders(...){}
    QC_SQLSTATEMENT->addMethod("bindPlaceholders", (q_method_n_t)SQLStatement_bindPlaceholders_VV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing SQLStatement::bindPlaceholdersArgs(softlist vargs){}
    QC_SQLSTATEMENT->addMethod("bindPlaceholdersArgs", (q_method_n_t)SQLStatement_bindPlaceholdersArgs_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softListTypeInfo, NULL, "vargs");

    // nothing SQLStatement::bindValues(...){}
    QC_SQLSTATEMENT->addMethod("bindValues", (q_method_n_t)SQLStatement_bindValues_VV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing SQLStatement::bindValuesArgs(softlist vargs){}
    QC_SQLSTATEMENT->addMethod("bindValuesArgs", (q_method_n_t)SQLStatement_bindValuesArgs_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softListTypeInfo, NULL, "vargs");

    // nothing SQLStatement::close(){}
    QC_SQLSTATEMENT->addMethod("close", (q_method_n_t)SQLStatement_close, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing SQLStatement::commit(){}
    QC_SQLSTATEMENT->addMethod("commit", (q_method_n_t)SQLStatement_commit, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // SQLStatement::constructor(Datasource ds) {}
    QC_SQLSTATEMENT->addConstructor(SQLStatement_constructor_C10Datasource, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, QC_DATASOURCE->getTypeInfo(), NULL, "ds");

    // SQLStatement::constructor(DatasourcePool dsp) {}
    QC_SQLSTATEMENT->addConstructor(SQLStatement_constructor_C14DatasourcePool, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, QC_DATASOURCEPOOL->getTypeInfo(), NULL, "dsp");

    // SQLStatement::copy() {}
    QC_SQLSTATEMENT->setCopy((q_copy_t)SQLStatement_copy);

    // bool SQLStatement::currentThreadInTransaction(){}
    QC_SQLSTATEMENT->addMethod("currentThreadInTransaction", (q_method_n_t)SQLStatement_currentThreadInTransaction, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // nothing SQLStatement::define(){}
    QC_SQLSTATEMENT->addMethod("define", (q_method_n_t)SQLStatement_define, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // hash SQLStatement::describe(){}
    QC_SQLSTATEMENT->addMethod("describe", (q_method_n_t)SQLStatement_describe, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo);

    // SQLStatement::destructor() {}
    QC_SQLSTATEMENT->setDestructor((q_destructor_t)SQLStatement_destructor);

    // nothing SQLStatement::exec(...){}
    QC_SQLSTATEMENT->addMethod("exec", (q_method_n_t)SQLStatement_exec_VV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing SQLStatement::execArgs(softlist vargs){}
    QC_SQLSTATEMENT->addMethod("execArgs", (q_method_n_t)SQLStatement_execArgs_vl, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softListTypeInfo, NULL, "vargs");

    // hash SQLStatement::fetchColumns(softint rows = -1){}
    QC_SQLSTATEMENT->addMethod("fetchColumns", (q_method_n_t)SQLStatement_fetchColumns_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo, 1, softBigIntTypeInfo, new QoreBigIntNode(-1), "rows");

    // *hash SQLStatement::fetchRow(){}
    QC_SQLSTATEMENT->addMethod("fetchRow", (q_method_n_t)SQLStatement_fetchRow, false, QC_NO_FLAGS, QDOM_DEFAULT, hashOrNothingTypeInfo);

    // list SQLStatement::fetchRows(softint rows = -1){}
    QC_SQLSTATEMENT->addMethod("fetchRows", (q_method_n_t)SQLStatement_fetchRows_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, listTypeInfo, 1, softBigIntTypeInfo, new QoreBigIntNode(-1), "rows");

    // hash SQLStatement::getOutput(){}
    QC_SQLSTATEMENT->addMethod("getOutput", (q_method_n_t)SQLStatement_getOutput, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo);

    // hash SQLStatement::getOutputRows(){}
    QC_SQLSTATEMENT->addMethod("getOutputRows", (q_method_n_t)SQLStatement_getOutputRows, false, QC_NO_FLAGS, QDOM_DEFAULT, hashTypeInfo);

    // *string SQLStatement::getSQL(){}
    QC_SQLSTATEMENT->addMethod("getSQL", (q_method_n_t)SQLStatement_getSQL, false, QC_NO_FLAGS, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *hash SQLStatement::getValue(){}
    QC_SQLSTATEMENT->addMethod("getValue", (q_method_n_t)SQLStatement_getValue, false, QC_NO_FLAGS, QDOM_DEFAULT, hashOrNothingTypeInfo);

    // auto SQLStatement::memberGate(string key){}
    QC_SQLSTATEMENT->addMethod("memberGate", (q_method_n_t)SQLStatement_memberGate_Vs, false, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, NULL, "key");

    // bool SQLStatement::next(){}
    QC_SQLSTATEMENT->addMethod("next", (q_method_n_t)SQLStatement_next, false, QC_NO_FLAGS, QDOM_DEFAULT, boolTypeInfo);

    // nothing SQLStatement::prepare(string sql, ...){}
    QC_SQLSTATEMENT->addMethod("prepare", (q_method_n_t)SQLStatement_prepare_VsVV, false, QC_USES_EXTRA_ARGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // nothing SQLStatement::prepareRaw(string sql){}
    QC_SQLSTATEMENT->addMethod("prepareRaw", (q_method_n_t)SQLStatement_prepareRaw_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, NULL, "sql");

    // nothing SQLStatement::rollback(){}
    QC_SQLSTATEMENT->addMethod("rollback", (q_method_n_t)SQLStatement_rollback, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // bool SQLStatement::valid(){}
    QC_SQLSTATEMENT->addMethod("valid", (q_method_n_t)SQLStatement_valid, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    return QC_SQLSTATEMENT;
}
