/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_SSLCertificate.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2015 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_SSLCertificate.h"
/* Qore class Qore::SSLCertificate */

qore_classid_t CID_SSLCERTIFICATE;
QoreClass* QC_SSLCERTIFICATE;

// SSLCertificate::constructor(string pem) {}
static void SSLCertificate_constructor_Vs(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pem = HARD_QORE_VALUE_STRING(args, 0);
# 53 "QC_SSLCertificate.qpp"
// create assuming a certificate in PEM format
   SimpleRefHolder<QoreSSLCertificate> qc(new QoreSSLCertificate(pem, xsink));
   if (!*xsink)
      self->setPrivate(CID_SSLCERTIFICATE, qc.release());
}

// SSLCertificate::constructor(binary der) {}
static void SSLCertificate_constructor_Vy(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* der = HARD_QORE_VALUE_BINARY(args, 0);
# 70 "QC_SSLCertificate.qpp"
SimpleRefHolder<QoreSSLCertificate> qc(new QoreSSLCertificate(der, xsink));
   if (!*xsink)
      self->setPrivate(CID_SSLCERTIFICATE, qc.release());

}

// SSLCertificate::copy() {}
static void SSLCertificate_copy(QoreObject* self, QoreObject* old, QoreSSLCertificate* s, ExceptionSink* xsink) {
# 80 "QC_SSLCertificate.qpp"
xsink->raiseException("SSLCERTIFICATE-COPY-ERROR", "SSLCertificate objects cannot be copied");
}

// hash SSLCertificate::getInfo(){}
static QoreValue SSLCertificate_getInfo(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 116 "QC_SSLCertificate.qpp"
    return s->getInfo();
}

// hash SSLCertificate::getIssuerHash(){}
static QoreValue SSLCertificate_getIssuerHash(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 188 "QC_SSLCertificate.qpp"
    return s->getIssuerHash();
}

// date SSLCertificate::getNotAfterDate(){}
static QoreValue SSLCertificate_getNotAfterDate(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 248 "QC_SSLCertificate.qpp"
    return s->getNotAfterDate();
}

// date SSLCertificate::getNotBeforeDate(){}
static QoreValue SSLCertificate_getNotBeforeDate(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 236 "QC_SSLCertificate.qpp"
    return s->getNotBeforeDate();
}

// string SSLCertificate::getPEM(){}
static QoreValue SSLCertificate_getPEM(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 94 "QC_SSLCertificate.qpp"
    return s->getPEM(xsink);
}

// *binary SSLCertificate::getPublicKey(){}
static QoreValue SSLCertificate_getPublicKey(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 152 "QC_SSLCertificate.qpp"
    return s->getPublicKey();
}

// string SSLCertificate::getPublicKeyAlgorithm(){}
static QoreValue SSLCertificate_getPublicKeyAlgorithm(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 164 "QC_SSLCertificate.qpp"
    return s->getPublicKeyAlgorithm();
}

// hash SSLCertificate::getPurposeHash(){}
static QoreValue SSLCertificate_getPurposeHash(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 224 "QC_SSLCertificate.qpp"
    return s->getPurposeHash();
}

// int SSLCertificate::getSerialNumber(){}
static QoreValue SSLCertificate_getSerialNumber(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 200 "QC_SSLCertificate.qpp"
    return s->getSerialNumber();
}

// binary SSLCertificate::getSignature(){}
static QoreValue SSLCertificate_getSignature(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 128 "QC_SSLCertificate.qpp"
    return s->getSignature();
}

// string SSLCertificate::getSignatureType(){}
static QoreValue SSLCertificate_getSignatureType(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 140 "QC_SSLCertificate.qpp"
    return s->getSignatureType();
}

// hash SSLCertificate::getSubjectHash(){}
static QoreValue SSLCertificate_getSubjectHash(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 176 "QC_SSLCertificate.qpp"
    return s->getSubjectHash();
}

// int SSLCertificate::getVersion(){}
static QoreValue SSLCertificate_getVersion(QoreObject* self, QoreSSLCertificate* s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 212 "QC_SSLCertificate.qpp"
    return s->getVersion();
}

DLLLOCAL void preinitSSLCertificateClass() {
    QC_SSLCERTIFICATE = new QoreClass("SSLCertificate", QDOM_DEFAULT);
    CID_SSLCERTIFICATE = QC_SSLCERTIFICATE->getID();
    QC_SSLCERTIFICATE->setSystem();
}

DLLLOCAL QoreClass* initSSLCertificateClass(QoreNamespace& ns) {
    if (!QC_SSLCERTIFICATE)
        preinitSSLCertificateClass();

    // SSLCertificate::constructor(string pem) {}
    QC_SSLCERTIFICATE->addConstructor(SSLCertificate_constructor_Vs, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, NULL, "pem");

    // SSLCertificate::constructor(binary der) {}
    QC_SSLCERTIFICATE->addConstructor(SSLCertificate_constructor_Vy, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, binaryTypeInfo, NULL, "der");

    // SSLCertificate::copy() {}
    QC_SSLCERTIFICATE->setCopy((q_copy_t)SSLCertificate_copy);

    // hash SSLCertificate::getInfo(){}
    QC_SSLCERTIFICATE->addMethod("getInfo", (q_method_n_t)SSLCertificate_getInfo, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // hash SSLCertificate::getIssuerHash(){}
    QC_SSLCERTIFICATE->addMethod("getIssuerHash", (q_method_n_t)SSLCertificate_getIssuerHash, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // date SSLCertificate::getNotAfterDate(){}
    QC_SSLCERTIFICATE->addMethod("getNotAfterDate", (q_method_n_t)SSLCertificate_getNotAfterDate, false, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo);

    // date SSLCertificate::getNotBeforeDate(){}
    QC_SSLCERTIFICATE->addMethod("getNotBeforeDate", (q_method_n_t)SSLCertificate_getNotBeforeDate, false, QC_CONSTANT, QDOM_DEFAULT, dateTypeInfo);

    // string SSLCertificate::getPEM(){}
    QC_SSLCERTIFICATE->addMethod("getPEM", (q_method_n_t)SSLCertificate_getPEM, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // *binary SSLCertificate::getPublicKey(){}
    QC_SSLCERTIFICATE->addMethod("getPublicKey", (q_method_n_t)SSLCertificate_getPublicKey, false, QC_CONSTANT, QDOM_DEFAULT, binaryOrNothingTypeInfo);

    // string SSLCertificate::getPublicKeyAlgorithm(){}
    QC_SSLCERTIFICATE->addMethod("getPublicKeyAlgorithm", (q_method_n_t)SSLCertificate_getPublicKeyAlgorithm, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // hash SSLCertificate::getPurposeHash(){}
    QC_SSLCERTIFICATE->addMethod("getPurposeHash", (q_method_n_t)SSLCertificate_getPurposeHash, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // int SSLCertificate::getSerialNumber(){}
    QC_SSLCERTIFICATE->addMethod("getSerialNumber", (q_method_n_t)SSLCertificate_getSerialNumber, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // binary SSLCertificate::getSignature(){}
    QC_SSLCERTIFICATE->addMethod("getSignature", (q_method_n_t)SSLCertificate_getSignature, false, QC_CONSTANT, QDOM_DEFAULT, binaryTypeInfo);

    // string SSLCertificate::getSignatureType(){}
    QC_SSLCERTIFICATE->addMethod("getSignatureType", (q_method_n_t)SSLCertificate_getSignatureType, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // hash SSLCertificate::getSubjectHash(){}
    QC_SSLCERTIFICATE->addMethod("getSubjectHash", (q_method_n_t)SSLCertificate_getSubjectHash, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // int SSLCertificate::getVersion(){}
    QC_SSLCERTIFICATE->addMethod("getVersion", (q_method_n_t)SSLCertificate_getVersion, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_SSLCERTIFICATE;
}
