/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_SSLPrivateKey.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2015 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_SSLPrivateKey.h"

#include <openssl/ssl.h>
#include <openssl/pem.h>
/* Qore class Qore::SSLPrivateKey */

qore_classid_t CID_SSLPRIVATEKEY;
QoreClass* QC_SSLPRIVATEKEY;

// SSLPrivateKey::constructor(string pem, *string pass) {}
static void SSLPrivateKey_constructor_VsNs(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pem = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* pass = get_param_value(args, 1).get<const QoreStringNode>();
# 57 "QC_SSLPrivateKey.qpp"
SimpleRefHolder<QoreSSLPrivateKey> qpk(new QoreSSLPrivateKey(pem, pass ? pass->getBuffer() : 0, xsink));

   if (!*xsink)
      self->setPrivate(CID_SSLPRIVATEKEY, qpk.release());
}

// SSLPrivateKey::constructor(binary der) {}
static void SSLPrivateKey_constructor_Vy(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const BinaryNode* der = HARD_QORE_VALUE_BINARY(args, 0);
# 74 "QC_SSLPrivateKey.qpp"
SimpleRefHolder<QoreSSLPrivateKey> qc(new QoreSSLPrivateKey(der, xsink));
   if (!*xsink)
      self->setPrivate(CID_SSLPRIVATEKEY, qc.release());
}

// SSLPrivateKey::copy() {}
static void SSLPrivateKey_copy(QoreObject* self, QoreObject* old, QoreSSLPrivateKey* pk, ExceptionSink* xsink) {
# 84 "QC_SSLPrivateKey.qpp"
xsink->raiseException("SSLPRIVATEKEY-COPY-ERROR", "SSLPrivateKey objects cannot be copied");
}

// int SSLPrivateKey::getBitLength(){}
static QoreValue SSLPrivateKey_getBitLength(QoreObject* self, QoreSSLPrivateKey* pk, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 132 "QC_SSLPrivateKey.qpp"
    return pk->getBitLength();
}

// hash SSLPrivateKey::getInfo(){}
static QoreValue SSLPrivateKey_getInfo(QoreObject* self, QoreSSLPrivateKey* pk, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 99 "QC_SSLPrivateKey.qpp"
    return pk->getInfo();
}

// string SSLPrivateKey::getPEM(){}
static QoreValue SSLPrivateKey_getPEM(QoreObject* self, QoreSSLPrivateKey* pk, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 146 "QC_SSLPrivateKey.qpp"
    return pk->getPEM(xsink);
}

// string SSLPrivateKey::getType(){}
static QoreValue SSLPrivateKey_getType(QoreObject* self, QoreSSLPrivateKey* pk, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 111 "QC_SSLPrivateKey.qpp"
    return new QoreStringNode(pk->getType());
}

// int SSLPrivateKey::getVersion(){}
static QoreValue SSLPrivateKey_getVersion(QoreObject* self, QoreSSLPrivateKey* pk, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 120 "QC_SSLPrivateKey.qpp"
    return pk->getVersion();
}

DLLLOCAL void preinitSSLPrivateKeyClass() {
    QC_SSLPRIVATEKEY = new QoreClass("SSLPrivateKey", QDOM_DEFAULT);
    CID_SSLPRIVATEKEY = QC_SSLPRIVATEKEY->getID();
    QC_SSLPRIVATEKEY->setSystem();
}

DLLLOCAL QoreClass* initSSLPrivateKeyClass(QoreNamespace& ns) {
    if (!QC_SSLPRIVATEKEY)
        preinitSSLPrivateKeyClass();

    // SSLPrivateKey::constructor(string pem, *string pass) {}
    QC_SSLPRIVATEKEY->addConstructor(SSLPrivateKey_constructor_VsNs, false, QC_NO_FLAGS, QDOM_DEFAULT, 2, stringTypeInfo, NULL, "pem", stringOrNothingTypeInfo, NULL, "pass");

    // SSLPrivateKey::constructor(binary der) {}
    QC_SSLPRIVATEKEY->addConstructor(SSLPrivateKey_constructor_Vy, false, QC_NO_FLAGS, QDOM_DEFAULT, 1, binaryTypeInfo, NULL, "der");

    // SSLPrivateKey::copy() {}
    QC_SSLPRIVATEKEY->setCopy((q_copy_t)SSLPrivateKey_copy);

    // int SSLPrivateKey::getBitLength(){}
    QC_SSLPRIVATEKEY->addMethod("getBitLength", (q_method_n_t)SSLPrivateKey_getBitLength, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // hash SSLPrivateKey::getInfo(){}
    QC_SSLPRIVATEKEY->addMethod("getInfo", (q_method_n_t)SSLPrivateKey_getInfo, false, QC_CONSTANT, QDOM_DEFAULT, hashTypeInfo);

    // string SSLPrivateKey::getPEM(){}
    QC_SSLPRIVATEKEY->addMethod("getPEM", (q_method_n_t)SSLPrivateKey_getPEM, false, QC_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo);

    // string SSLPrivateKey::getType(){}
    QC_SSLPRIVATEKEY->addMethod("getType", (q_method_n_t)SSLPrivateKey_getType, false, QC_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // int SSLPrivateKey::getVersion(){}
    QC_SSLPRIVATEKEY->addMethod("getVersion", (q_method_n_t)SSLPrivateKey_getVersion, false, QC_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    return QC_SSLPRIVATEKEY;
}
