/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  QC_TermIOS.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2015 David Nichols

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_TermIOS.h"
/* Qore class Qore::TermIOS */

qore_classid_t CID_TERMIOS;
QoreClass* QC_TERMIOS;

// TermIOS::constructor() {}
static void TermIOS_constructor(QoreObject* self, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 586 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   self->setPrivate(CID_TERMIOS, new QoreTermIOS);
#else
   missing_method_error("TermIOS::constructor", "TERMIOS", xsink);
#endif
}

// TermIOS::copy() {}
static void TermIOS_copy(QoreObject* self, QoreObject* old, QoreTermIOS*s, ExceptionSink* xsink) {
# 603 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   self->setPrivate(CID_TERMIOS, new QoreTermIOS(*s));
#else
   missing_method_error("TermIOS::copy", "TERMIOS", xsink);
#endif
}

// int TermIOS::getCC(softint cc){}
static QoreValue TermIOS_getCC_vi(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 cc = HARD_QORE_VALUE_INT(args, 0);
# 801 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   return s->get_cc(cc, xsink);
#else
   return missing_method_error("TermIOS::getCC", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// int TermIOS::getCFlag(){}
static QoreValue TermIOS_getCFlag(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 641 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   return s->get_cflag();
#else
   return missing_method_error("TermIOS::getCFlag", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// int TermIOS::getIFlag(){}
static QoreValue TermIOS_getIFlag(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 679 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   return s->get_iflag();
#else
   return missing_method_error("TermIOS::getIFlag", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// int TermIOS::getLFlag(){}
static QoreValue TermIOS_getLFlag(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 622 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   return s->get_lflag();
#else
   return missing_method_error("TermIOS::getLFlag", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// int TermIOS::getOFlag(){}
static QoreValue TermIOS_getOFlag(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 660 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   return s->get_oflag();
#else
   return missing_method_error("TermIOS::getOFlag", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// bool TermIOS::isEqual(Termios termios){}
static QoreValue TermIOS_isEqual_C7Termios(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(termios, QoreTermIOS, args, 0, CID_TERMIOS, "TermIOS::isEqual()", "Termios", xsink);
    if (*xsink)
        return 0;
# 847 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   ReferenceHolder<QoreTermIOS> holder(termios, xsink);
   return s->is_equal(termios);
#else
   termios = 0;
   return missing_method_error("TermIOS::isEqual", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// nothing TermIOS::setCC(softint offset, softint value){}
static QoreValue TermIOS_setCC_vivi(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 offset = HARD_QORE_VALUE_INT(args, 0);
    int64 value = HARD_QORE_VALUE_INT(args, 1);
# 827 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   s->set_cc(offset, (cc_t)value, xsink);
   return QoreValue();
#else
   return missing_method_error("TermIOS::setCC", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// nothing TermIOS::setCFlag(softint flag){}
static QoreValue TermIOS_setCFlag_vi(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 flag = HARD_QORE_VALUE_INT(args, 0);
# 728 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   s->set_cflag((int)flag);
   return QoreValue();
#else
   return missing_method_error("TermIOS::setCFlag", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// nothing TermIOS::setIFlag(softint flag){}
static QoreValue TermIOS_setIFlag_vi(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 flag = HARD_QORE_VALUE_INT(args, 0);
# 774 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   s->set_iflag((int)flag);
   return QoreValue();
#else
   return missing_method_error("TermIOS::setIFlag", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// nothing TermIOS::setLFlag(softint flag){}
static QoreValue TermIOS_setLFlag_vi(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 flag = HARD_QORE_VALUE_INT(args, 0);
# 705 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   s->set_lflag((int)flag);
   return QoreValue();
#else
   return missing_method_error("TermIOS::setLFlag", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// nothing TermIOS::setOFlag(softint flag){}
static QoreValue TermIOS_setOFlag_vi(QoreObject* self, QoreTermIOS*s, const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 flag = HARD_QORE_VALUE_INT(args, 0);
# 751 "QC_TermIOS.qpp"
    #ifdef HAVE_TERMIOS_H
   s->set_oflag((int)flag);
   return QoreValue();
#else
   return missing_method_error("TermIOS::setOFlag", "TERMIOS", xsink);
#endif
    return QoreValue();
}

// static hash TermIOS::getWindowSize(){}
static QoreValue static_TermIOS_getWindowSize(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 869 "QC_TermIOS.qpp"
#ifdef HAVE_TERMIOS_H
   int rows, columns;

   if (QoreTermIOS::getWindowSize(rows, columns, xsink))
      return QoreValue();

   QoreHashNode *rv = new QoreHashNode;
   rv->setKeyValue("rows", new QoreBigIntNode(rows), xsink);
   rv->setKeyValue("columns", new QoreBigIntNode(columns), xsink);
   return rv;
#else
   return missing_method_error("TermIOS::getWindowSize", "TERMIOS", xsink);
#endif
    return QoreValue();
}

DLLLOCAL void preinitTermIOSClass() {
    QC_TERMIOS = new QoreClass("TermIOS", QDOM_TERMINAL_IO);
    CID_TERMIOS = QC_TERMIOS->getID();
    QC_TERMIOS->setSystem();
}

DLLLOCAL QoreClass* initTermIOSClass(QoreNamespace& ns) {
    if (!QC_TERMIOS)
        preinitTermIOSClass();

    // TermIOS::constructor() {}
    QC_TERMIOS->addConstructor(TermIOS_constructor, false, QC_NO_FLAGS, QDOM_DEFAULT);

    // TermIOS::copy() {}
    QC_TERMIOS->setCopy((q_copy_t)TermIOS_copy);

    // int TermIOS::getCC(softint cc){}
    QC_TERMIOS->addMethod("getCC", (q_method_n_t)TermIOS_getCC_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo, 1, softBigIntTypeInfo, NULL, "cc");

    // int TermIOS::getCFlag(){}
    QC_TERMIOS->addMethod("getCFlag", (q_method_n_t)TermIOS_getCFlag, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int TermIOS::getIFlag(){}
    QC_TERMIOS->addMethod("getIFlag", (q_method_n_t)TermIOS_getIFlag, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int TermIOS::getLFlag(){}
    QC_TERMIOS->addMethod("getLFlag", (q_method_n_t)TermIOS_getLFlag, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // int TermIOS::getOFlag(){}
    QC_TERMIOS->addMethod("getOFlag", (q_method_n_t)TermIOS_getOFlag, false, QC_NO_FLAGS, QDOM_DEFAULT, bigIntTypeInfo);

    // bool TermIOS::isEqual(Termios termios){}
    QC_TERMIOS->addMethod("isEqual", (q_method_n_t)TermIOS_isEqual_C7Termios, false, QC_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_TERMIOS->getTypeInfo(), NULL, "termios");

    // nothing TermIOS::setCC(softint offset, softint value){}
    QC_TERMIOS->addMethod("setCC", (q_method_n_t)TermIOS_setCC_vivi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, softBigIntTypeInfo, NULL, "offset", softBigIntTypeInfo, NULL, "value");

    // nothing TermIOS::setCFlag(softint flag){}
    QC_TERMIOS->addMethod("setCFlag", (q_method_n_t)TermIOS_setCFlag_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "flag");

    // nothing TermIOS::setIFlag(softint flag){}
    QC_TERMIOS->addMethod("setIFlag", (q_method_n_t)TermIOS_setIFlag_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "flag");

    // nothing TermIOS::setLFlag(softint flag){}
    QC_TERMIOS->addMethod("setLFlag", (q_method_n_t)TermIOS_setLFlag_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "flag");

    // nothing TermIOS::setOFlag(softint flag){}
    QC_TERMIOS->addMethod("setOFlag", (q_method_n_t)TermIOS_setOFlag_vi, false, QC_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, NULL, "flag");

    // static hash TermIOS::getWindowSize(){}
    QC_TERMIOS->addStaticMethod("getWindowSize", (q_func_n_t)static_TermIOS_getWindowSize, false, QC_RET_VALUE_ONLY, QDOM_TERMINAL_IO, hashTypeInfo);
# 36 "QC_TermIOS.qpp"
#ifndef ECHOKE
#define ECHOKE 0
#endif
# 43 "QC_TermIOS.qpp"
#ifndef ECHOE
#define ECHOE 0
#endif
# 49 "QC_TermIOS.qpp"
#ifndef ECHO
#define ECHO 0
#endif
# 55 "QC_TermIOS.qpp"
#ifndef ECHONL
#define ECHONL 0
#endif
# 61 "QC_TermIOS.qpp"
#ifndef ECHOPRT
#define ECHOPRT 0
#endif
# 67 "QC_TermIOS.qpp"
#ifndef ECHOCTL
#define ECHOCTL 0
#endif
# 73 "QC_TermIOS.qpp"
#ifndef ISIG
#define ISIG 0
#endif
# 79 "QC_TermIOS.qpp"
#ifndef ICANON
#define ICANON 0
#endif
# 85 "QC_TermIOS.qpp"
#ifndef ALTWERASE
#define ALTWERASE 0
#endif
# 91 "QC_TermIOS.qpp"
#ifndef IEXTEN
#define IEXTEN 0
#endif
# 97 "QC_TermIOS.qpp"
#ifndef EXTPROC
#define EXTPROC 0
#endif
# 103 "QC_TermIOS.qpp"
#ifndef TOSTOP
#define TOSTOP 0
#endif
# 109 "QC_TermIOS.qpp"
#ifndef FLUSHO
#define FLUSHO 0
#endif
# 115 "QC_TermIOS.qpp"
#ifndef NOKERNINFO
#define NOKERNINFO 0
#endif
# 121 "QC_TermIOS.qpp"
#ifndef PENDIN
#define PENDIN 0
#endif
# 127 "QC_TermIOS.qpp"
#ifndef NOFLSH
#define NOFLSH 0
#endif
    ns.addConstant("ALTWERASE", new QoreBigIntNode(ALTWERASE));
    ns.addConstant("ECHO", new QoreBigIntNode(ECHO));
    ns.addConstant("ECHOCTL", new QoreBigIntNode(ECHOCTL));
    ns.addConstant("ECHOE", new QoreBigIntNode(ECHOE));
    ns.addConstant("ECHOKE", new QoreBigIntNode(ECHOKE));
    ns.addConstant("ECHONL", new QoreBigIntNode(ECHONL));
    ns.addConstant("ECHOPRT", new QoreBigIntNode(ECHOPRT));
    ns.addConstant("EXTPROC", new QoreBigIntNode(EXTPROC));
    ns.addConstant("FLUSHO", new QoreBigIntNode(FLUSHO));
    ns.addConstant("ICANON", new QoreBigIntNode(ICANON));
    ns.addConstant("IEXTEN", new QoreBigIntNode(IEXTEN));
    ns.addConstant("ISIG", new QoreBigIntNode(ISIG));
    ns.addConstant("NOFLSH", new QoreBigIntNode(NOFLSH));
    ns.addConstant("NOKERNINFO", new QoreBigIntNode(NOKERNINFO));
    ns.addConstant("PENDIN", new QoreBigIntNode(PENDIN));
    ns.addConstant("TOSTOP", new QoreBigIntNode(TOSTOP));
# 137 "QC_TermIOS.qpp"
#ifndef CSIZE
#define CSIZE 0
#endif
# 144 "QC_TermIOS.qpp"
#ifndef CS5
#define CS5 0
#endif
# 150 "QC_TermIOS.qpp"
#ifndef CS6
#define CS6 0
#endif
# 156 "QC_TermIOS.qpp"
#ifndef CS7
#define CS7 0
#endif
# 162 "QC_TermIOS.qpp"
#ifndef CS8
#define CS8 0
#endif
# 168 "QC_TermIOS.qpp"
#ifndef CSTOPB
#define CSTOPB 0
#endif
# 174 "QC_TermIOS.qpp"
#ifndef CREAD
#define CREAD 0
#endif
# 180 "QC_TermIOS.qpp"
#ifndef PARENB
#define PARENB 0
#endif
# 186 "QC_TermIOS.qpp"
#ifndef PARODD
#define PARODD 0
#endif
# 192 "QC_TermIOS.qpp"
#ifndef HUPCL
#define HUPCL 0
#endif
# 198 "QC_TermIOS.qpp"
#ifndef CLOCAL
#define CLOCAL 0
#endif
# 204 "QC_TermIOS.qpp"
#ifndef CCTS_OFLOW
#define CCTS_OFLOW 0
#endif
# 210 "QC_TermIOS.qpp"
#ifndef CRTSCTS
#define CRTSCTS 0
#endif
# 216 "QC_TermIOS.qpp"
#ifndef CRTS_IFLOW
#define CRTS_IFLOW 0
#endif
# 222 "QC_TermIOS.qpp"
#ifndef CDSR_OFLOW
#define CDSR_OFLOW 0
#endif
# 228 "QC_TermIOS.qpp"
#ifndef CCAR_OFLOW
#define CCAR_OFLOW 0
#endif
# 234 "QC_TermIOS.qpp"
#ifndef MDMBUF
#define MDMBUF 0
#endif
    ns.addConstant("CCAR_OFLOW", new QoreBigIntNode(CCAR_OFLOW));
    ns.addConstant("CCTS_OFLOW", new QoreBigIntNode(CCTS_OFLOW));
    ns.addConstant("CDSR_OFLOW", new QoreBigIntNode(CDSR_OFLOW));
    ns.addConstant("CLOCAL", new QoreBigIntNode(CLOCAL));
    ns.addConstant("CREAD", new QoreBigIntNode(CREAD));
    ns.addConstant("CRTSCTS", new QoreBigIntNode(CRTSCTS));
    ns.addConstant("CRTS_IFLOW", new QoreBigIntNode(CRTS_IFLOW));
    ns.addConstant("CS5", new QoreBigIntNode(CS5));
    ns.addConstant("CS6", new QoreBigIntNode(CS6));
    ns.addConstant("CS7", new QoreBigIntNode(CS7));
    ns.addConstant("CS8", new QoreBigIntNode(CS8));
    ns.addConstant("CSIZE", new QoreBigIntNode(CSIZE));
    ns.addConstant("CSTOPB", new QoreBigIntNode(CSTOPB));
    ns.addConstant("HUPCL", new QoreBigIntNode(HUPCL));
    ns.addConstant("MDMBUF", new QoreBigIntNode(MDMBUF));
    ns.addConstant("PARENB", new QoreBigIntNode(PARENB));
    ns.addConstant("PARODD", new QoreBigIntNode(PARODD));
# 244 "QC_TermIOS.qpp"
#ifndef OPOST
#define OPOST 0
#endif
# 251 "QC_TermIOS.qpp"
#ifndef ONLCR
#define ONLCR 0
#endif
# 257 "QC_TermIOS.qpp"
#ifndef OXTABS
#define OXTABS 0
#endif
# 263 "QC_TermIOS.qpp"
#ifndef ONOEOT
#define ONOEOT 0
#endif
# 269 "QC_TermIOS.qpp"
#ifndef OCRNL
#define OCRNL 0
#endif
# 275 "QC_TermIOS.qpp"
#ifndef OLCUC
#define OLCUC 0
#endif
# 281 "QC_TermIOS.qpp"
#ifndef ONOCR
#define ONOCR 0
#endif
# 287 "QC_TermIOS.qpp"
#ifndef ONLRET
#define ONLRET 0
#endif
    ns.addConstant("OCRNL", new QoreBigIntNode(OCRNL));
    ns.addConstant("OLCUC", new QoreBigIntNode(OLCUC));
    ns.addConstant("ONLCR", new QoreBigIntNode(ONLCR));
    ns.addConstant("ONLRET", new QoreBigIntNode(ONLRET));
    ns.addConstant("ONOCR", new QoreBigIntNode(ONOCR));
    ns.addConstant("ONOEOT", new QoreBigIntNode(ONOEOT));
    ns.addConstant("OPOST", new QoreBigIntNode(OPOST));
    ns.addConstant("OXTABS", new QoreBigIntNode(OXTABS));
# 297 "QC_TermIOS.qpp"
#ifndef IGNBRK
#define IGNBRK 0
#endif
# 304 "QC_TermIOS.qpp"
#ifndef BRKINT
#define BRKINT 0
#endif
# 310 "QC_TermIOS.qpp"
#ifndef IGNPAR
#define IGNPAR 0
#endif
# 316 "QC_TermIOS.qpp"
#ifndef PARMRK
#define PARMRK 0
#endif
# 322 "QC_TermIOS.qpp"
#ifndef INPCK
#define INPCK 0
#endif
# 328 "QC_TermIOS.qpp"
#ifndef ISTRIP
#define ISTRIP 0
#endif
# 334 "QC_TermIOS.qpp"
#ifndef INLCR
#define INLCR 0
#endif
# 340 "QC_TermIOS.qpp"
#ifndef IGNCR
#define IGNCR 0
#endif
# 346 "QC_TermIOS.qpp"
#ifndef ICRNL
#define ICRNL 0
#endif
# 352 "QC_TermIOS.qpp"
#ifndef IXON
#define IXON 0
#endif
# 358 "QC_TermIOS.qpp"
#ifndef IXOFF
#define IXOFF 0
#endif
# 364 "QC_TermIOS.qpp"
#ifndef IXANY
#define IXANY 0
#endif
# 370 "QC_TermIOS.qpp"
#ifndef IMAXBEL
#define IMAXBEL 0
#endif
# 376 "QC_TermIOS.qpp"
#ifndef IUCLC
#define IUCLC 0
#endif
    ns.addConstant("BRKINT", new QoreBigIntNode(BRKINT));
    ns.addConstant("ICRNL", new QoreBigIntNode(ICRNL));
    ns.addConstant("IGNBRK", new QoreBigIntNode(IGNBRK));
    ns.addConstant("IGNCR", new QoreBigIntNode(IGNCR));
    ns.addConstant("IGNPAR", new QoreBigIntNode(IGNPAR));
    ns.addConstant("IMAXBEL", new QoreBigIntNode(IMAXBEL));
    ns.addConstant("INLCR", new QoreBigIntNode(INLCR));
    ns.addConstant("INPCK", new QoreBigIntNode(INPCK));
    ns.addConstant("ISTRIP", new QoreBigIntNode(ISTRIP));
    ns.addConstant("IUCLC", new QoreBigIntNode(IUCLC));
    ns.addConstant("IXANY", new QoreBigIntNode(IXANY));
    ns.addConstant("IXOFF", new QoreBigIntNode(IXOFF));
    ns.addConstant("IXON", new QoreBigIntNode(IXON));
    ns.addConstant("PARMRK", new QoreBigIntNode(PARMRK));
# 386 "QC_TermIOS.qpp"
#ifndef VEOF
#define VEOF 0
#endif
# 393 "QC_TermIOS.qpp"
#ifndef VEOL
#define VEOL 0
#endif
# 399 "QC_TermIOS.qpp"
#ifndef VEOL2
#define VEOL2 0
#endif
# 405 "QC_TermIOS.qpp"
#ifndef VERASE
#define VERASE 0
#endif
# 411 "QC_TermIOS.qpp"
#ifndef VWERASE
#define VWERASE 0
#endif
# 417 "QC_TermIOS.qpp"
#ifndef VKILL
#define VKILL 0
#endif
# 423 "QC_TermIOS.qpp"
#ifndef VREPRINT
#define VREPRINT 0
#endif
# 429 "QC_TermIOS.qpp"
#ifndef VINTR
#define VINTR 0
#endif
# 435 "QC_TermIOS.qpp"
#ifndef VQUIT
#define VQUIT 0
#endif
# 441 "QC_TermIOS.qpp"
#ifndef VSUSP
#define VSUSP 0
#endif
# 447 "QC_TermIOS.qpp"
#ifndef VDSUSP
#define VDSUSP 0
#endif
# 453 "QC_TermIOS.qpp"
#ifndef VSTART
#define VSTART 0
#endif
# 459 "QC_TermIOS.qpp"
#ifndef VSTOP
#define VSTOP 0
#endif
# 465 "QC_TermIOS.qpp"
#ifndef VLNEXT
#define VLNEXT 0
#endif
# 471 "QC_TermIOS.qpp"
#ifndef VDISCARD
#define VDISCARD 0
#endif
# 477 "QC_TermIOS.qpp"
#ifndef VMIN
#define VMIN 0
#endif
# 483 "QC_TermIOS.qpp"
#ifndef VTIME
#define VTIME 0
#endif
# 489 "QC_TermIOS.qpp"
#ifndef VSTATUS
#define VSTATUS 0
#endif
# 495 "QC_TermIOS.qpp"
#ifndef _POSIX_VDISABLE
#define _POSIX_VDISABLE 0xff
#endif
    ns.addConstant("VDISCARD", new QoreBigIntNode(VDISCARD));
    ns.addConstant("VDSUSP", new QoreBigIntNode(VDSUSP));
    ns.addConstant("VEOF", new QoreBigIntNode(VEOF));
    ns.addConstant("VEOL", new QoreBigIntNode(VEOL));
    ns.addConstant("VEOL2", new QoreBigIntNode(VEOL2));
    ns.addConstant("VERASE", new QoreBigIntNode(VERASE));
    ns.addConstant("VINTR", new QoreBigIntNode(VINTR));
    ns.addConstant("VKILL", new QoreBigIntNode(VKILL));
    ns.addConstant("VLNEXT", new QoreBigIntNode(VLNEXT));
    ns.addConstant("VMIN", new QoreBigIntNode(VMIN));
    ns.addConstant("VQUIT", new QoreBigIntNode(VQUIT));
    ns.addConstant("VREPRINT", new QoreBigIntNode(VREPRINT));
    ns.addConstant("VSTART", new QoreBigIntNode(VSTART));
    ns.addConstant("VSTATUS", new QoreBigIntNode(VSTATUS));
    ns.addConstant("VSTOP", new QoreBigIntNode(VSTOP));
    ns.addConstant("VSUSP", new QoreBigIntNode(VSUSP));
    ns.addConstant("VTIME", new QoreBigIntNode(VTIME));
    ns.addConstant("VWERASE", new QoreBigIntNode(VWERASE));
    ns.addConstant("_POSIX_VDISABLE", new QoreBigIntNode(_POSIX_VDISABLE));
# 505 "QC_TermIOS.qpp"
#ifndef TCSANOW
#define TCSANOW 0
#endif
# 512 "QC_TermIOS.qpp"
#ifndef TCSADRAIN
#define TCSADRAIN 0
#endif
# 518 "QC_TermIOS.qpp"
#ifndef TCSAFLUSH
#define TCSAFLUSH 0
#endif
# 524 "QC_TermIOS.qpp"
#ifndef TCSASOFT
#define TCSASOFT 0
#endif
    ns.addConstant("TCSADRAIN", new QoreBigIntNode(TCSADRAIN));
    ns.addConstant("TCSAFLUSH", new QoreBigIntNode(TCSAFLUSH));
    ns.addConstant("TCSANOW", new QoreBigIntNode(TCSANOW));
    ns.addConstant("TCSASOFT", new QoreBigIntNode(TCSASOFT));

    return QC_TERMIOS;
}
