/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  qc_qore.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2016 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_File.h"

DLLEXPORT extern QoreClass* QC_FILEINPUTSTREAM;
DLLEXPORT extern QoreClass* QC_FILEOUTPUTSTREAM;

#ifdef WORDS_BIGENDIAN
#define Q_MACHINE_MSB 1
#else
#define Q_MACHINE_MSB 0
#endif

#ifdef _Q_WINDOWS
#define Q_IS_WINDOWS 1
#define Q_IS_UNIX    0
#else
#define Q_IS_WINDOWS 0
#define Q_IS_UNIX    1
#endif

DLLLOCAL void init_qore_constants(QoreNamespace& ns) {
# 53 "qc_qore.qpp"
# 60 "qc_qore.qpp"
# 63 "qc_qore.qpp"
# 66 "qc_qore.qpp"
# 69 "qc_qore.qpp"
# 72 "qc_qore.qpp"
# 75 "qc_qore.qpp"
# 78 "qc_qore.qpp"
# 81 "qc_qore.qpp"
# 84 "qc_qore.qpp"
# 87 "qc_qore.qpp"
# 90 "qc_qore.qpp"
# 93 "qc_qore.qpp"
    ns.addConstant("NT_BINARY", new QoreBigIntNode(NT_BINARY));
    ns.addConstant("NT_BOOLEAN", new QoreBigIntNode(NT_BOOLEAN));
    ns.addConstant("NT_CALLREF", new QoreBigIntNode(NT_FUNCREF));
    ns.addConstant("NT_CLOSURE", new QoreBigIntNode(NT_RUNTIME_CLOSURE));
    ns.addConstant("NT_DATE", new QoreBigIntNode(NT_DATE));
    ns.addConstant("NT_FLOAT", new QoreBigIntNode(NT_FLOAT));
    ns.addConstant("NT_HASH", new QoreBigIntNode(NT_HASH));
    ns.addConstant("NT_INT", new QoreBigIntNode(NT_INT));
    ns.addConstant("NT_LIST", new QoreBigIntNode(NT_LIST));
    ns.addConstant("NT_NOTHING", new QoreBigIntNode(NT_NOTHING));
    ns.addConstant("NT_NULL", new QoreBigIntNode(NT_NULL));
    ns.addConstant("NT_NUMBER", new QoreBigIntNode(NT_NUMBER));
    ns.addConstant("NT_OBJECT", new QoreBigIntNode(NT_OBJECT));
    ns.addConstant("NT_STRING", new QoreBigIntNode(NT_STRING));
# 100 "qc_qore.qpp"
# 123 "qc_qore.qpp"
    QoreHashNode* hash_TypeCodeMap = new QoreHashNode;
    hash_TypeCodeMap->setKeyValue("0", new QoreStringNode("nothing"), 0);
    hash_TypeCodeMap->setKeyValue("1", new QoreStringNode("integer"), 0);
    hash_TypeCodeMap->setKeyValue("2", new QoreStringNode("float"), 0);
    hash_TypeCodeMap->setKeyValue("3", new QoreStringNode("string"), 0);
    hash_TypeCodeMap->setKeyValue("4", new QoreStringNode("date"), 0);
    hash_TypeCodeMap->setKeyValue("5", new QoreStringNode("bool"), 0);
    hash_TypeCodeMap->setKeyValue("6", new QoreStringNode("NULL"), 0);
    hash_TypeCodeMap->setKeyValue("7", new QoreStringNode("binary"), 0);
    hash_TypeCodeMap->setKeyValue("8", new QoreStringNode("list"), 0);
    hash_TypeCodeMap->setKeyValue("9", new QoreStringNode("hash"), 0);
    hash_TypeCodeMap->setKeyValue("10", new QoreStringNode("object"), 0);
    hash_TypeCodeMap->setKeyValue("11", new QoreStringNode("number"), 0);
    hash_TypeCodeMap->setKeyValue("29", new QoreStringNode("call reference"), 0);
    hash_TypeCodeMap->setKeyValue("32", new QoreStringNode("closure"), 0);

    ns.addConstant("TypeCodeMap", hash_TypeCodeMap);
    QoreHashNode* hash_TypeNameMap = new QoreHashNode;
    hash_TypeNameMap->setKeyValue("nothing", new QoreBigIntNode(NT_NOTHING), 0);
    hash_TypeNameMap->setKeyValue("integer", new QoreBigIntNode(NT_INT), 0);
    hash_TypeNameMap->setKeyValue("float", new QoreBigIntNode(NT_FLOAT), 0);
    hash_TypeNameMap->setKeyValue("number", new QoreBigIntNode(NT_NUMBER), 0);
    hash_TypeNameMap->setKeyValue("string", new QoreBigIntNode(NT_STRING), 0);
    hash_TypeNameMap->setKeyValue("date", new QoreBigIntNode(NT_DATE), 0);
    hash_TypeNameMap->setKeyValue("bool", new QoreBigIntNode(NT_BOOLEAN), 0);
    hash_TypeNameMap->setKeyValue("NULL", new QoreBigIntNode(NT_NULL), 0);
    hash_TypeNameMap->setKeyValue("binary", new QoreBigIntNode(NT_BINARY), 0);
    hash_TypeNameMap->setKeyValue("list", new QoreBigIntNode(NT_LIST), 0);
    hash_TypeNameMap->setKeyValue("hash", new QoreBigIntNode(NT_HASH), 0);
    hash_TypeNameMap->setKeyValue("object", new QoreBigIntNode(NT_OBJECT), 0);
    hash_TypeNameMap->setKeyValue("call reference", new QoreBigIntNode(NT_FUNCREF), 0);
    hash_TypeNameMap->setKeyValue("closure", new QoreBigIntNode(NT_RUNTIME_CLOSURE), 0);

    ns.addConstant("TypeNameMap", hash_TypeNameMap);
# 142 "qc_qore.qpp"
    ns.addConstant("False", get_bool_node(false));
    ns.addConstant("True", get_bool_node(true));
# 152 "qc_qore.qpp"
    ns.addConstant("NOTHING", (&Nothing));
    ns.addConstant("NULL", (&Null));
# 162 "qc_qore.qpp"
    ns.addConstant("ET_System", new QoreStringNode("System"));
    ns.addConstant("ET_User", new QoreStringNode("User"));
# 172 "qc_qore.qpp"
# 178 "qc_qore.qpp"
# 181 "qc_qore.qpp"
    ns.addConstant("CT_Builtin", new QoreBigIntNode(CT_BUILTIN));
    ns.addConstant("CT_NewThread", new QoreBigIntNode(CT_NEWTHREAD));
    ns.addConstant("CT_Rethrow", new QoreBigIntNode(CT_RETHROW));
    ns.addConstant("CT_User", new QoreBigIntNode(CT_USER));
# 188 "qc_qore.qpp"
# 194 "qc_qore.qpp"
# 197 "qc_qore.qpp"
# 200 "qc_qore.qpp"
# 203 "qc_qore.qpp"
# 206 "qc_qore.qpp"
# 209 "qc_qore.qpp"
# 212 "qc_qore.qpp"
# 215 "qc_qore.qpp"
# 218 "qc_qore.qpp"
# 221 "qc_qore.qpp"
# 224 "qc_qore.qpp"
# 227 "qc_qore.qpp"
# 230 "qc_qore.qpp"
    ns.addConstant("Build", (new QoreBigIntNode(qore_build_number)));
    ns.addConstant("BuildHost", (new QoreStringNode(qore_build_host)));
    ns.addConstant("CFLAGS", (new QoreStringNode(qore_cflags)));
    ns.addConstant("Compiler", (new QoreStringNode(qore_cplusplus_compiler)));
    ns.addConstant("DirSep", (new QoreStringNode(QORE_DIR_SEP_STR)));
    ns.addConstant("LDFLAGS", (new QoreStringNode(qore_ldflags)));
    ns.addConstant("MACHINE_MSB", get_bool_node(Q_MACHINE_MSB));
    ns.addConstant("PathSep", (new QoreStringNode(QORE_PATH_SEP_STR)));
    QoreHashNode* hash_Platform = new QoreHashNode;
    hash_Platform->setKeyValue("OS", (new QoreStringNode(TARGET_OS)), 0);
    hash_Platform->setKeyValue("CPU", (new QoreStringNode(TARGET_ARCH)), 0);
    hash_Platform->setKeyValue("Unix", get_bool_node(Q_IS_UNIX), 0);
    hash_Platform->setKeyValue("Windows", get_bool_node(Q_IS_WINDOWS), 0);

    ns.addConstant("Platform", hash_Platform);
    ns.addConstant("PlatformCPU", (new QoreStringNode(TARGET_ARCH)));
    ns.addConstant("PlatformOS", (new QoreStringNode(TARGET_OS)));
    ns.addConstant("VersionMajor", (new QoreBigIntNode(qore_version_major)));
    ns.addConstant("VersionMinor", (new QoreBigIntNode(qore_version_minor)));
    ns.addConstant("VersionString", (new QoreStringNode(qore_version_string)));
    ns.addConstant("VersionSub", (new QoreBigIntNode(qore_version_sub)));
# 247 "qc_qore.qpp"
# 253 "qc_qore.qpp"
# 256 "qc_qore.qpp"
    ns.addConstant("SOURCE_FILE", new QoreBigIntNode(QORE_SOURCE_FILE));
    ns.addConstant("SOURCE_FTPCLIENT", new QoreBigIntNode(QORE_SOURCE_FTPCLIENT));
    ns.addConstant("SOURCE_HTTPCLIENT", new QoreBigIntNode(QORE_SOURCE_HTTPCLIENT));
    ns.addConstant("SOURCE_SOCKET", new QoreBigIntNode(QORE_SOURCE_SOCKET));
# 263 "qc_qore.qpp"
# 274 "qc_qore.qpp"
    QoreHashNode* hash_EVENT_MAP = new QoreHashNode;
    hash_EVENT_MAP->setKeyValue("1", new QoreStringNode("PACKET_READ"), 0);
    hash_EVENT_MAP->setKeyValue("2", new QoreStringNode("PACKET_SENT"), 0);
    hash_EVENT_MAP->setKeyValue("3", new QoreStringNode("HTTP_CONTENT_LENGTH"), 0);
    hash_EVENT_MAP->setKeyValue("4", new QoreStringNode("HTTP_CHUNKED_START"), 0);
    hash_EVENT_MAP->setKeyValue("5", new QoreStringNode("HTTP_CHUNKED_END"), 0);
    hash_EVENT_MAP->setKeyValue("6", new QoreStringNode("HTTP_REDIRECT"), 0);
    hash_EVENT_MAP->setKeyValue("7", new QoreStringNode("CHANNEL_CLOSED"), 0);
    hash_EVENT_MAP->setKeyValue("8", new QoreStringNode("DELETED"), 0);
    hash_EVENT_MAP->setKeyValue("9", new QoreStringNode("FTP_SEND_MESSAGE"), 0);
    hash_EVENT_MAP->setKeyValue("10", new QoreStringNode("FTP_MESSAGE_RECEIVED"), 0);
    hash_EVENT_MAP->setKeyValue("11", new QoreStringNode("HOSTNAME_LOOKUP"), 0);
    hash_EVENT_MAP->setKeyValue("12", new QoreStringNode("HOSTNAME_RESOLVED"), 0);
    hash_EVENT_MAP->setKeyValue("13", new QoreStringNode("HTTP_SEND_MESSAGE"), 0);
    hash_EVENT_MAP->setKeyValue("14", new QoreStringNode("HTTP_MESSAGE_RECEIVED"), 0);
    hash_EVENT_MAP->setKeyValue("15", new QoreStringNode("HTTP_FOOTERS_RECEIVED"), 0);
    hash_EVENT_MAP->setKeyValue("16", new QoreStringNode("HTTP_CHUNKED_DATA_RECEIVED"), 0);
    hash_EVENT_MAP->setKeyValue("17", new QoreStringNode("HTTP_CHUNK_SIZE"), 0);
    hash_EVENT_MAP->setKeyValue("18", new QoreStringNode("CONNECTING"), 0);
    hash_EVENT_MAP->setKeyValue("19", new QoreStringNode("CONNECTED"), 0);
    hash_EVENT_MAP->setKeyValue("20", new QoreStringNode("START_SSL"), 0);
    hash_EVENT_MAP->setKeyValue("21", new QoreStringNode("SSL_ESTABLISHED"), 0);
    hash_EVENT_MAP->setKeyValue("22", new QoreStringNode("OPEN_FILE"), 0);
    hash_EVENT_MAP->setKeyValue("23", new QoreStringNode("FILE_OPENED"), 0);
    hash_EVENT_MAP->setKeyValue("24", new QoreStringNode("DATA_READ"), 0);
    hash_EVENT_MAP->setKeyValue("25", new QoreStringNode("DATA_WRITTEN"), 0);

    ns.addConstant("EVENT_MAP", hash_EVENT_MAP);
    QoreHashNode* hash_EVENT_SOURCE_MAP = new QoreHashNode;
    hash_EVENT_SOURCE_MAP->setKeyValue("1", new QoreStringNode("SOCKET"), 0);
    hash_EVENT_SOURCE_MAP->setKeyValue("2", new QoreStringNode("HTTPCLIENT"), 0);
    hash_EVENT_SOURCE_MAP->setKeyValue("3", new QoreStringNode("FTPCLIENT"), 0);
    hash_EVENT_SOURCE_MAP->setKeyValue("4", new QoreStringNode("FILE"), 0);

    ns.addConstant("EVENT_SOURCE_MAP", hash_EVENT_SOURCE_MAP);
# 304 "qc_qore.qpp"
# 310 "qc_qore.qpp"
# 313 "qc_qore.qpp"
# 316 "qc_qore.qpp"
# 319 "qc_qore.qpp"
# 322 "qc_qore.qpp"
# 325 "qc_qore.qpp"
# 328 "qc_qore.qpp"
# 331 "qc_qore.qpp"
# 334 "qc_qore.qpp"
# 337 "qc_qore.qpp"
# 340 "qc_qore.qpp"
# 343 "qc_qore.qpp"
# 346 "qc_qore.qpp"
# 349 "qc_qore.qpp"
# 352 "qc_qore.qpp"
# 355 "qc_qore.qpp"
# 358 "qc_qore.qpp"
# 361 "qc_qore.qpp"
# 364 "qc_qore.qpp"
# 367 "qc_qore.qpp"
# 370 "qc_qore.qpp"
# 373 "qc_qore.qpp"
# 376 "qc_qore.qpp"
    ns.addConstant("EVENT_CHANNEL_CLOSED", new QoreBigIntNode(QORE_EVENT_CHANNEL_CLOSED));
    ns.addConstant("EVENT_CONNECTED", new QoreBigIntNode(QORE_EVENT_CONNECTED));
    ns.addConstant("EVENT_CONNECTING", new QoreBigIntNode(QORE_EVENT_CONNECTING));
    ns.addConstant("EVENT_DATA_READ", new QoreBigIntNode(QORE_EVENT_DATA_READ));
    ns.addConstant("EVENT_DATA_WRITTEN", new QoreBigIntNode(QORE_EVENT_DATA_WRITTEN));
    ns.addConstant("EVENT_DELETED", new QoreBigIntNode(QORE_EVENT_DELETED));
    ns.addConstant("EVENT_FILE_OPENED", new QoreBigIntNode(QORE_EVENT_FILE_OPENED));
    ns.addConstant("EVENT_FTP_MESSAGE_RECEIVED", new QoreBigIntNode(QORE_EVENT_FTP_MESSAGE_RECEIVED));
    ns.addConstant("EVENT_FTP_SEND_MESSAGE", new QoreBigIntNode(QORE_EVENT_FTP_SEND_MESSAGE));
    ns.addConstant("EVENT_HOSTNAME_LOOKUP", new QoreBigIntNode(QORE_EVENT_HOSTNAME_LOOKUP));
    ns.addConstant("EVENT_HOSTNAME_RESOLVED", new QoreBigIntNode(QORE_EVENT_HOSTNAME_RESOLVED));
    ns.addConstant("EVENT_HTTP_CHUNKED_DATA_RECEIVED", new QoreBigIntNode(QORE_EVENT_HTTP_CHUNKED_DATA_RECEIVED));
    ns.addConstant("EVENT_HTTP_CHUNKED_END", new QoreBigIntNode(QORE_EVENT_HTTP_CHUNKED_END));
    ns.addConstant("EVENT_HTTP_CHUNKED_START", new QoreBigIntNode(QORE_EVENT_HTTP_CHUNKED_START));
    ns.addConstant("EVENT_HTTP_CHUNK_SIZE", new QoreBigIntNode(QORE_EVENT_HTTP_CHUNK_SIZE));
    ns.addConstant("EVENT_HTTP_CONTENT_LENGTH", new QoreBigIntNode(QORE_EVENT_HTTP_CONTENT_LENGTH));
    ns.addConstant("EVENT_HTTP_FOOTERS_RECEIVED", new QoreBigIntNode(QORE_EVENT_HTTP_FOOTERS_RECEIVED));
    ns.addConstant("EVENT_HTTP_MESSAGE_RECEIVED", new QoreBigIntNode(QORE_EVENT_HTTP_MESSAGE_RECEIVED));
    ns.addConstant("EVENT_HTTP_REDIRECT", new QoreBigIntNode(QORE_EVENT_HTTP_REDIRECT));
    ns.addConstant("EVENT_HTTP_SEND_MESSAGE", new QoreBigIntNode(QORE_EVENT_HTTP_SEND_MESSAGE));
    ns.addConstant("EVENT_OPEN_FILE", new QoreBigIntNode(QORE_EVENT_OPEN_FILE));
    ns.addConstant("EVENT_PACKET_READ", new QoreBigIntNode(QORE_EVENT_PACKET_READ));
    ns.addConstant("EVENT_PACKET_SENT", new QoreBigIntNode(QORE_EVENT_PACKET_SENT));
    ns.addConstant("EVENT_SSL_ESTABLISHED", new QoreBigIntNode(QORE_EVENT_SSL_ESTABLISHED));
    ns.addConstant("EVENT_START_SSL", new QoreBigIntNode(QORE_EVENT_START_SSL));
# 383 "qc_qore.qpp"
# 392 "qc_qore.qpp"
# 395 "qc_qore.qpp"
# 398 "qc_qore.qpp"
# 401 "qc_qore.qpp"
    ns.addConstant("stderr", (QC_FILE->execSystemConstructor(2)));
    ns.addConstant("stderr_stream", (QC_FILEOUTPUTSTREAM->execSystemConstructor(2)));
    ns.addConstant("stdin", (QC_FILE->execSystemConstructor(0)));
    ns.addConstant("stdin_stream", (QC_FILEINPUTSTREAM->execSystemConstructor(0)));
    ns.addConstant("stdout", (QC_FILE->execSystemConstructor(1)));
    ns.addConstant("stdout_stream", (QC_FILEOUTPUTSTREAM->execSystemConstructor(1)));

}
