/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
  ql_list.qpp

  Qore Programming Language

  Copyright (C) 2003 - 2017 Qore Technologies, s.r.o.

  Permission is hereby granted, free of charge, to any person obtaining a
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

  Note that the Qore library is released under a choice of three open-source
  licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
  information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_list.h"
#include "qore/intern/qore_program_private.h"

ResolvedCallReferenceNode* getCallReference(const QoreString* str, ExceptionSink* xsink) {
   // ensure string is in default encoding
   TempEncodingHelper tmp(str, QCS_DEFAULT, xsink);
   if (!tmp)
      return 0;
   return qore_program_private::runtimeGetCallReference(getProgram(), tmp->getBuffer(), xsink);
}

bool inlist_intern(const QoreValue arg, const QoreListNode* l, ExceptionSink* xsink) {
   assert(xsink);
   ConstListIterator li(l);
   while (li.next()) {
      bool b = QoreLogicalEqualsOperatorNode::softEqual(arg, li.getValue(), xsink);
      if (*xsink)
         return false;
      if (b)
         return true;
   }
   return false;
}

QoreListNode* range_intern(int64 start, int64 stop, int64 step, ExceptionSink* xsink) {
    if (step < 1) {
        xsink->raiseException("RANGE-ERROR", "Value of the 'step' argument has to be greater than 0");
        return 0;
    }

    QoreListNode* l = new QoreListNode;
    if (start < stop) {
        for (int64 i = start; i <= stop; i += step) {
            l->push(new QoreBigIntNode(i));
        }
    }
    else {
        for (int64 i = start; i >= stop; i -= step) {
            l->push(new QoreBigIntNode(i));
        }
    }
    return l;
}
# 78 "ql_list.qpp"
# 89 "ql_list.qpp"
# 110 "ql_list.qpp"
# 132 "ql_list.qpp"
# 154 "ql_list.qpp"
# 161 "ql_list.qpp"
# 179 "ql_list.qpp"
# 198 "ql_list.qpp"
# 216 "ql_list.qpp"
# 223 "ql_list.qpp"
# 246 "ql_list.qpp"
# 270 "ql_list.qpp"
# 294 "ql_list.qpp"
# 301 "ql_list.qpp"
# 319 "ql_list.qpp"
# 338 "ql_list.qpp"
# 362 "ql_list.qpp"
# 369 "ql_list.qpp"
# 392 "ql_list.qpp"
# 416 "ql_list.qpp"
# 440 "ql_list.qpp"
# 447 "ql_list.qpp"
# 470 "ql_list.qpp"
# 494 "ql_list.qpp"
# 518 "ql_list.qpp"
# 525 "ql_list.qpp"
# 548 "ql_list.qpp"
# 572 "ql_list.qpp"
# 596 "ql_list.qpp"
# 614 "ql_list.qpp"
# 633 "ql_list.qpp"
# 652 "ql_list.qpp"
# 670 "ql_list.qpp"
# 688 "ql_list.qpp"
# 707 "ql_list.qpp"
# 726 "ql_list.qpp"
# 744 "ql_list.qpp"
# 750 "ql_list.qpp"
# 767 "ql_list.qpp"
# 774 "ql_list.qpp"
# 781 "ql_list.qpp"
# 800 "ql_list.qpp"
# 807 "ql_list.qpp"
# 814 "ql_list.qpp"
# 855 "ql_list.qpp"
# 885 "ql_list.qpp"
// bool inlist() {}
static QoreValue f_inlist(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 770 "ql_list.qpp"
   return false;
}

// bool inlist(any arg, nothing x) {}
static QoreValue f_inlist_VaVn(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 777 "ql_list.qpp"
   return false;
}

// bool inlist(auto arg, softlist l) {}
static QoreValue f_inlist_VAvl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 1);
# 796 "ql_list.qpp"
   return inlist_intern(arg, l, xsink);
}

// bool inlist_hard() {}
static QoreValue f_inlist_hard(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 803 "ql_list.qpp"
   return false;
}

// bool inlist_hard(any arg, nothing x) {}
static QoreValue f_inlist_hard_VaVn(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 810 "ql_list.qpp"
   return false;
}

// bool inlist_hard(auto arg, softlist l) {}
static QoreValue f_inlist_hard_VAvl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 1);
# 829 "ql_list.qpp"
   bool arg_is_nothing = arg.isNothing();

   ConstListIterator li(l);
   while (li.next()) {
      const AbstractQoreNode* lp = li.getValue();

      bool b;

      // do hard comparison inline
      if (is_nothing(lp))
	 b = arg_is_nothing;
      else if (arg_is_nothing)
	 b = false;
      else {
	 b = arg.isEqualHard(lp);
	 if (*xsink)
	    return QoreValue();
      }

      if (b)
	 return true;
   }
   return false;
}

// auto max(list l) {}
static QoreValue f_max_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 684 "ql_list.qpp"
   return l->max();
}

// auto max(list l, string func) {}
static QoreValue f_max_VlVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 702 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->max(*fr, xsink);
}

// auto max(list l, code f) {}
static QoreValue f_max_VlVc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 722 "ql_list.qpp"
   return l->max(f, xsink);
}

// auto max(...) {}
static QoreValue f_max_VV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 740 "ql_list.qpp"
   return args ? args->maxValue(xsink) : 0;
}

// auto min(list l) {}
static QoreValue f_min_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 610 "ql_list.qpp"
   return l->min();
}

// auto min(list l, string func) {}
static QoreValue f_min_VlVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 628 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->min(*fr, xsink);
}

// auto min(list l, code f) {}
static QoreValue f_min_VlVc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 648 "ql_list.qpp"
   return l->min(f, xsink);
}

// auto min(...) {}
static QoreValue f_min_VV(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 666 "ql_list.qpp"
   return args ? args->minValue(xsink) : 0;
}

// list range(int start, int stop, int step = 1) {}
static QoreValue f_range_ViViVi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 start = HARD_QORE_VALUE_INT(args, 0);
    int64 stop = HARD_QORE_VALUE_INT(args, 1);
    int64 step = HARD_QORE_VALUE_INT(args, 2);
# 881 "ql_list.qpp"
    return range_intern(start, stop, step, xsink);
}

// list range(int stop) {}
static QoreValue f_range_Vi(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 stop = HARD_QORE_VALUE_INT(args, 0);
# 905 "ql_list.qpp"
    return range_intern(0, stop, 1, xsink);
}

// nothing reverse() {}
static QoreValue f_reverse(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 747 "ql_list.qpp"

    return QoreValue();
}

// list reverse(list l) {}
static QoreValue f_reverse_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 763 "ql_list.qpp"
   return l->reverse();
}

// auto sort(auto arg) {}
static QoreValue f_sort_VA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 85 "ql_list.qpp"
   return arg.getReferencedValue();
}

// list sort(list l) {}
static QoreValue f_sort_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 106 "ql_list.qpp"
   return l->sort();
}

// list sort(list l, string func) {}
static QoreValue f_sort_VlVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 127 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sort(*fr, xsink);
}

// list sort(list l, code f) {}
static QoreValue f_sort_VlVc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 150 "ql_list.qpp"
   return l->sort(f, xsink);
}

// auto sortDescending(auto arg) {}
static QoreValue f_sortDescending_VA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 157 "ql_list.qpp"
   return arg.getReferencedValue();
}

// list sortDescending(list l) {}
static QoreValue f_sortDescending_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 175 "ql_list.qpp"
   return l->sortDescending();
}

// list sortDescending(list l, string func) {}
static QoreValue f_sortDescending_VlVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 193 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sortDescending(*fr, xsink);
}

// list sortDescending(list l, code f) {}
static QoreValue f_sortDescending_VlVc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 212 "ql_list.qpp"
   return l->sortDescending(f, xsink);
}

// auto sortDescendingStable(auto arg) {}
static QoreValue f_sortDescendingStable_VA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 443 "ql_list.qpp"
   return arg.getReferencedValue();
}

// list sortDescendingStable(list l) {}
static QoreValue f_sortDescendingStable_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 466 "ql_list.qpp"
   return l->sortDescendingStable();
}

// list sortDescendingStable(list l, string func) {}
static QoreValue f_sortDescendingStable_VlVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 489 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sortDescendingStable(*fr, xsink);
}

// list sortDescendingStable(list l, code f) {}
static QoreValue f_sortDescendingStable_VlVc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 514 "ql_list.qpp"
   return l->sortDescendingStable(f, xsink);
}

// auto sortStable(auto arg) {}
static QoreValue f_sortStable_VA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 297 "ql_list.qpp"
   return arg.getReferencedValue();
}

// list sortStable(list l) {}
static QoreValue f_sortStable_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 315 "ql_list.qpp"
   return l->sortStable();
}

// list sortStable(list l, string func) {}
static QoreValue f_sortStable_VlVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 333 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sortStable(*fr, xsink);
}

// list sortStable(list l, code f) {}
static QoreValue f_sortStable_VlVc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 358 "ql_list.qpp"
   return l->sortStable(f, xsink);
}

// auto sort_descending(auto arg) {}
static QoreValue f_sort_descending_VA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 219 "ql_list.qpp"
   return arg.getReferencedValue();
}

// list sort_descending(list l) {}
static QoreValue f_sort_descending_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 242 "ql_list.qpp"
   return l->sortDescending();
}

// list sort_descending(list l, string func) {}
static QoreValue f_sort_descending_VlVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 265 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sortDescending(*fr, xsink);
}

// list sort_descending(list l, code f) {}
static QoreValue f_sort_descending_VlVc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 290 "ql_list.qpp"
   return l->sortDescending(f, xsink);
}

// auto sort_descending_stable(auto arg) {}
static QoreValue f_sort_descending_stable_VA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 521 "ql_list.qpp"
   return arg.getReferencedValue();
}

// list sort_descending_stable(list l) {}
static QoreValue f_sort_descending_stable_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 544 "ql_list.qpp"
   return l->sortDescendingStable();
}

// list sort_descending_stable(list l, string func) {}
static QoreValue f_sort_descending_stable_VlVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 567 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sortDescendingStable(*fr, xsink);
}

// list sort_descending_stable(list l, code f) {}
static QoreValue f_sort_descending_stable_VlVc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 592 "ql_list.qpp"
   return l->sortDescendingStable(f, xsink);
}

// auto sort_stable(auto arg) {}
static QoreValue f_sort_stable_VA(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue arg = get_param_value(args, 0);
# 365 "ql_list.qpp"
   return arg.getReferencedValue();
}

// list sort_stable(list l) {}
static QoreValue f_sort_stable_Vl(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
# 388 "ql_list.qpp"
   return l->sortStable();
}

// list sort_stable(list l, string func) {}
static QoreValue f_sort_stable_VlVs(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const QoreStringNode* func = HARD_QORE_VALUE_STRING(args, 1);
# 411 "ql_list.qpp"
   ReferenceHolder<ResolvedCallReferenceNode> fr(getCallReference(func, xsink), xsink);
   return !fr ? 0 : l->sortStable(*fr, xsink);
}

// list sort_stable(list l, code f) {}
static QoreValue f_sort_stable_VlVc(const QoreValueList* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreListNode* l = HARD_QORE_VALUE_LIST(args, 0);
    const ResolvedCallReferenceNode* f = get_param_value(args, 1).get<const ResolvedCallReferenceNode>();
# 436 "ql_list.qpp"
   return l->sortStable(f, xsink);
}


DLLLOCAL void init_list_functions(QoreNamespace& ns) {
    // bool inlist() {}
    ns.addBuiltinVariant("inlist", (q_func_n_t)f_inlist, QC_NOOP, QDOM_DEFAULT, boolTypeInfo);
    // bool inlist(any arg, nothing x) {}
    ns.addBuiltinVariant("inlist", (q_func_n_t)f_inlist_VaVn, QC_NOOP, QDOM_DEFAULT, boolTypeInfo, 2, anyTypeInfo, NULL, "arg", nothingTypeInfo, NULL, "x");
    // bool inlist(auto arg, softlist l) {}
    ns.addBuiltinVariant("inlist", (q_func_n_t)f_inlist_VAvl, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 2, autoTypeInfo, NULL, "arg", softListTypeInfo, NULL, "l");
    // bool inlist_hard() {}
    ns.addBuiltinVariant("inlist_hard", (q_func_n_t)f_inlist_hard, QC_NOOP, QDOM_DEFAULT, boolTypeInfo);
    // bool inlist_hard(any arg, nothing x) {}
    ns.addBuiltinVariant("inlist_hard", (q_func_n_t)f_inlist_hard_VaVn, QC_NOOP, QDOM_DEFAULT, boolTypeInfo, 2, anyTypeInfo, NULL, "arg", nothingTypeInfo, NULL, "x");
    // bool inlist_hard(auto arg, softlist l) {}
    ns.addBuiltinVariant("inlist_hard", (q_func_n_t)f_inlist_hard_VAvl, QC_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 2, autoTypeInfo, NULL, "arg", softListTypeInfo, NULL, "l");
    // auto max(list l) {}
    ns.addBuiltinVariant("max", (q_func_n_t)f_max_Vl, QC_CONSTANT, QDOM_DEFAULT, autoTypeInfo, 1, listTypeInfo, NULL, "l");
    // auto max(list l, string func) {}
    ns.addBuiltinVariant("max", (q_func_n_t)f_max_VlVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
    // auto max(list l, code f) {}
    ns.addBuiltinVariant("max", (q_func_n_t)f_max_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
    // auto max(...) {}
    ns.addBuiltinVariant("max", (q_func_n_t)f_max_VV, QC_CONSTANT|QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo);
    // auto min(list l) {}
    ns.addBuiltinVariant("min", (q_func_n_t)f_min_Vl, QC_CONSTANT, QDOM_DEFAULT, autoTypeInfo, 1, listTypeInfo, NULL, "l");
    // auto min(list l, string func) {}
    ns.addBuiltinVariant("min", (q_func_n_t)f_min_VlVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
    // auto min(list l, code f) {}
    ns.addBuiltinVariant("min", (q_func_n_t)f_min_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
    // auto min(...) {}
    ns.addBuiltinVariant("min", (q_func_n_t)f_min_VV, QC_CONSTANT|QC_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo);
    // list range(int start, int stop, int step = 1) {}
    ns.addBuiltinVariant("range", (q_func_n_t)f_range_ViViVi, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 3, bigIntTypeInfo, NULL, "start", bigIntTypeInfo, NULL, "stop", bigIntTypeInfo, new QoreBigIntNode(1), "step");
    // list range(int stop) {}
    ns.addBuiltinVariant("range", (q_func_n_t)f_range_Vi, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, bigIntTypeInfo, NULL, "stop");
    // nothing reverse() {}
    ns.addBuiltinVariant("reverse", (q_func_n_t)f_reverse, QC_NOOP, QDOM_DEFAULT, nothingTypeInfo);
    // list reverse(list l) {}
    ns.addBuiltinVariant("reverse", (q_func_n_t)f_reverse_Vl, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
    // auto sort(auto arg) {}
    ns.addBuiltinVariant("sort", (q_func_n_t)f_sort_VA, QC_NOOP, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, NULL, "arg");
    // list sort(list l) {}
    ns.addBuiltinVariant("sort", (q_func_n_t)f_sort_Vl, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
    // list sort(list l, string func) {}
    ns.addBuiltinVariant("sort", (q_func_n_t)f_sort_VlVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
    // list sort(list l, code f) {}
    ns.addBuiltinVariant("sort", (q_func_n_t)f_sort_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
    // auto sortDescending(auto arg) {}
    ns.addBuiltinVariant("sortDescending", (q_func_n_t)f_sortDescending_VA, QC_NOOP|QC_DEPRECATED, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, NULL, "arg");
    // list sortDescending(list l) {}
    ns.addBuiltinVariant("sortDescending", (q_func_n_t)f_sortDescending_Vl, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
    // list sortDescending(list l, string func) {}
    ns.addBuiltinVariant("sortDescending", (q_func_n_t)f_sortDescending_VlVs, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
    // list sortDescending(list l, code f) {}
    ns.addBuiltinVariant("sortDescending", (q_func_n_t)f_sortDescending_VlVc, QC_RET_VALUE_ONLY|QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
    // auto sortDescendingStable(auto arg) {}
    ns.addBuiltinVariant("sortDescendingStable", (q_func_n_t)f_sortDescendingStable_VA, QC_NOOP|QC_DEPRECATED, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, NULL, "arg");
    // list sortDescendingStable(list l) {}
    ns.addBuiltinVariant("sortDescendingStable", (q_func_n_t)f_sortDescendingStable_Vl, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
    // list sortDescendingStable(list l, string func) {}
    ns.addBuiltinVariant("sortDescendingStable", (q_func_n_t)f_sortDescendingStable_VlVs, QC_RET_VALUE_ONLY|QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
    // list sortDescendingStable(list l, code f) {}
    ns.addBuiltinVariant("sortDescendingStable", (q_func_n_t)f_sortDescendingStable_VlVc, QC_RET_VALUE_ONLY|QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
    // auto sortStable(auto arg) {}
    ns.addBuiltinVariant("sortStable", (q_func_n_t)f_sortStable_VA, QC_NOOP|QC_DEPRECATED, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, NULL, "arg");
    // list sortStable(list l) {}
    ns.addBuiltinVariant("sortStable", (q_func_n_t)f_sortStable_Vl, QC_CONSTANT|QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
    // list sortStable(list l, string func) {}
    ns.addBuiltinVariant("sortStable", (q_func_n_t)f_sortStable_VlVs, QC_RET_VALUE_ONLY|QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
    // list sortStable(list l, code f) {}
    ns.addBuiltinVariant("sortStable", (q_func_n_t)f_sortStable_VlVc, QC_RET_VALUE_ONLY|QC_DEPRECATED, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
    // auto sort_descending(auto arg) {}
    ns.addBuiltinVariant("sort_descending", (q_func_n_t)f_sort_descending_VA, QC_NOOP, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, NULL, "arg");
    // list sort_descending(list l) {}
    ns.addBuiltinVariant("sort_descending", (q_func_n_t)f_sort_descending_Vl, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
    // list sort_descending(list l, string func) {}
    ns.addBuiltinVariant("sort_descending", (q_func_n_t)f_sort_descending_VlVs, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
    // list sort_descending(list l, code f) {}
    ns.addBuiltinVariant("sort_descending", (q_func_n_t)f_sort_descending_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
    // auto sort_descending_stable(auto arg) {}
    ns.addBuiltinVariant("sort_descending_stable", (q_func_n_t)f_sort_descending_stable_VA, QC_NOOP, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, NULL, "arg");
    // list sort_descending_stable(list l) {}
    ns.addBuiltinVariant("sort_descending_stable", (q_func_n_t)f_sort_descending_stable_Vl, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
    // list sort_descending_stable(list l, string func) {}
    ns.addBuiltinVariant("sort_descending_stable", (q_func_n_t)f_sort_descending_stable_VlVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
    // list sort_descending_stable(list l, code f) {}
    ns.addBuiltinVariant("sort_descending_stable", (q_func_n_t)f_sort_descending_stable_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");
    // auto sort_stable(auto arg) {}
    ns.addBuiltinVariant("sort_stable", (q_func_n_t)f_sort_stable_VA, QC_NOOP, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, NULL, "arg");
    // list sort_stable(list l) {}
    ns.addBuiltinVariant("sort_stable", (q_func_n_t)f_sort_stable_Vl, QC_CONSTANT, QDOM_DEFAULT, listTypeInfo, 1, listTypeInfo, NULL, "l");
    // list sort_stable(list l, string func) {}
    ns.addBuiltinVariant("sort_stable", (q_func_n_t)f_sort_stable_VlVs, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", stringTypeInfo, NULL, "func");
    // list sort_stable(list l, code f) {}
    ns.addBuiltinVariant("sort_stable", (q_func_n_t)f_sort_stable_VlVc, QC_RET_VALUE_ONLY, QDOM_DEFAULT, listTypeInfo, 2, listTypeInfo, NULL, "l", codeTypeInfo, NULL, "f");

}
